/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* COUNT2.H                                                  */
/*************************************************************/

#ifndef COUNT2_H
#define COUNT2_H

const MIN_ASCII_VALUE = 32;
const MAX_ASCII_VALUE = 126;
const ASCII_a = 97;


template<class Count_Type>
class Count {
public:

	// Constructors/destructors
	Count() { }
	virtual ~Count() { }

	// Implementors
	virtual void increment() = 0;
	virtual void decrement() = 0;
	void reset(){value = reset_value;}

	// Accessors
	Count_Type get_value() { return value; }
	void set_value(Count_Type new_value) { value =
		new_value; }

	Count_Type get_reset_value() { return reset_value; }
	void set_reset_value(Count_Type new_value) { value =
		new_value; }

protected:

	// Data members
	Count_Type value;
	Count_Type reset_value;
};

class Integer_Count : public Count<int> {
public:

	// Constructors/destructors
	Integer_Count() { reset_value = 0; reset(); }
	Integer_Count(int new_value)
		{ reset_value = 0; value = new_value; }
	~Integer_Count() { }

	// Implementors
	void increment();
	void decrement();
	char* asBase(int number_base);
};

class ASCII_Count : public Count<char> {
public:

	// Constructors/destructors
	ASCII_Count() { reset_value = 'a'; reset(); }
	ASCII_Count(char new_char) { reset_value = 'a';
				 value = new_char; }
	~ASCII_Count() { }

	// Implementors
	void increment();
	void decrement();
};

#endif
