/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* COUNT4.H                                                  */
/*************************************************************/

#ifndef COUNT4_H
#define COUNT4_H

#ifndef DATE_H
#include "date.h"
#endif

const MIN_ASCII_VALUE = 32;
const MAX_ASCII_VALUE = 126;
const ASCII_a = 97;

const MAX_SEQUENCE_SIZE = 10;

template<class Count_Type>
class Count {
public:

	// Constructors/destructors
	Count() { }
	virtual ~Count() { }

	// Implementors
	virtual void increment() = 0;
	virtual void decrement() = 0;
	void reset(){value = reset_value;}

	// Accessors
	Count_Type get_value() { return value; }
	void set_value(Count_Type new_value) { value =
		new_value; }

	Count_Type get_reset_value() { return reset_value; }
	void set_reset_value(Count_Type new_value) { value =
		new_value; }

	// Human interaction accessor
	Count_Type get_display_value() { return value; }

protected:

	// Data members
	Count_Type value;
	Count_Type reset_value;
};

class Integer_Count : public Count<int> {
public:

	// Constructors/destructors
	Integer_Count() { reset_value = 0; reset(); }
	Integer_Count(int new_value)
		{ reset_value = 0; value = new_value; }
	~Integer_Count() { }

	// Implementors
	void increment();
	void decrement();
	char* asBase(int number_base);
};

class ASCII_Count : public Count<char> {
public:

	// Constructors/destructors
	ASCII_Count() { reset_value = 'a'; reset(); }
	ASCII_Count(char new_char) { reset_value = 'a';
				 value = new_char; }
	~ASCII_Count() { }

	// Implementors
	void increment();
	void decrement();
};


class Date_Count : public Count<Date> {
public:

	// Constructors/destructors
	Date_Count() { this->reset(); }
	Date_Count(Date& new_date) {reset_value = new_date;
		this->reset(); }
	~Date_Count() { }

	// Implementors
	void increment();
	void decrement();

	// Helper member functions
	char* get_display_value();
};

class Sequence_Count : public Count<char*> {
public:

	// Constructors/destructors
	Sequence_Count() { position = -1; }
	Sequence_Count(char* new_sequence[]);
	Sequence_Count(char* new_sequence[], int sequence_size);
	~Sequence_Count() { }

	// Implementors
	void increment();
	void decrement();
	void reset();

	// Accessors
	char** get_sequence() { return sequence; }
	void set_sequence(char* new_sequence[]);

	// Helper member functions
	void initialize_reset_value();

private:

	// Data members
	char* sequence[MAX_SEQUENCE_SIZE];
	int position;
};



#endif
