/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* CVWCNTR2.H                                                */
/*************************************************************/

#ifndef COU_VWS2_H
#include "cou_vws2.h"
#endif

#ifndef CVWCNTR2_H
#define CVWCNTR2_H

template<class Count_Type, class Value_Type>
class Count_View_Container {
public:

	// Constructors/destructors
	Count_View_Container () { }
	~Count_View_Container () { }

	// Implementors
	void display_count_views(Value_Type initial_value);
};

// Template function bodies for Count_View_Container
// The compiler must see the actual definition of a template
// function, not just its prototype, in order to instantiate it.

template<class Count_Type, class Value_Type>
void Count_View_Container<Count_Type, Value_Type>
		::display_count_views(Value_Type initial_value)
{
	// Create and initialize the count.
	Count_Type count(initial_value);

	// Create the console menu and display box.
	Count_Console_Menu<Count_Type> menu;
	Count_Console_Display_Box<Count_Type> display_box;

	// Customize the menu for the count.
	menu.make_items(count);
	menu.add_terminator("Quit");
	menu.add_prompt("Make selection:  ");

	// Customize the display box for the count.
	display_box.set_count(count);
	display_box.set_label_string("Current Value:  " );

	// Display the count views.
	display_box.display_value();
	while (menu.get_user_selection() != 0)
		{
		menu.execute_selection();
		display_box.display_value();
		}
}

#endif