/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* VEND2.H                                                   */
/*************************************************************/

#include "string.h"
#ifndef BOOLEAN
#define BOOLEAN
enum Boolean { false, true };
#endif

#ifndef VEND2_H
#define VEND2_H

const MAX_ITEMS_SIZE = 10;

class Cash_Device {
public:

	// Constructors/destructors
	Cash_Device() { amount_collected = 0.0; }
	Cash_Device(float cash_amount)
		{ amount_collected = cash_amount; }
	~Cash_Device() { }

	// Implementors
	Boolean got_enough(float amount_to_collect);
	float return_cash();

	float make_change(float cash_amount);
	void add_cash(float cash_amount);

	// Accessors
	float get_amount_collected() { return amount_collected; }
	void set_amount_collected(float cash_amount)
		{ amount_collected = cash_amount; }

private:

	// Data members
	float amount_collected;
};

// Forward declarations
class Dispensing_Holder;

class Item {
public:

	// Constructors/destructors
	Item() { price = 0.0;}
	Item(String new_name, float new_price)
			{name = new_name; price = new_price;}
	~Item() { }

	// Implementors
	Boolean vend();

	// Accessors
	String& get_name() { return name; }
	void set_name(String& new_name) { name = new_name; }

	float get_price() { return price; }
	void set_price(float new_price) { price = new_price; }

	Cash_Device* get_cash_device() { return cash_device; }
	void connect_to_cash_device(Cash_Device& new_Device)
					{ cash_device = &new_Device; }

	Dispensing_Holder& get_dispensing_holder()
		{ return *a_dispensing_holder; }
	void connect_to_dispensing_holder
		(Dispensing_Holder& new_holder)
			{ a_dispensing_holder =  &new_holder; }

	// Overloaded operators for Items
	Boolean operator== (Item& an_item);
	friend ostream& operator<< (ostream& os, Item& an_item);


private:

	// Data members
	String name;
	float price;
	Cash_Device* cash_device;
	Dispensing_Holder* a_dispensing_holder;
};

class Dispensing_Holder {
public:

	// Constructors/destructors
	Dispensing_Holder() { item_index = 0; }
	~Dispensing_Holder() { }

	//Implementors: Testing
	Boolean is_empty();
	Boolean item_is_dispensable(Item& item_to_dispense);

	//Implementors: Dispensing
	Boolean dispense(Item& item_to_dispense);
	void activate_dispenser();
	void print_item(Item& item_to_dispense);

	// Accessors
	Item** get_items() { return items; }
	Boolean connect_to_item(Item& new_item);
	Boolean disconnect_with_item(Item& new_item);

private:

	// Data members
	Item* items[MAX_ITEMS_SIZE];
	int item_index;

};

#endif