/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* VEND4.CPP                                                 */
/*************************************************************/

#include <iostream.h>
#ifndef VEND4_H
#include "vend4.h"
#endif

// Cash_Device member functions

Boolean Cash_Device::got_enough(float amount_to_collect)
{
	if (amount_collected >= amount_to_collect)
		return true;
	else
		return false;
}

float Cash_Device::make_change(float item_price)
{
	amount_collected -= item_price;
	return amount_collected;

}

float Cash_Device::return_cash()
{
	float return_amount = amount_collected;
	amount_collected = 0;
	return return_amount;
}


void Cash_Device::add_cash(float cash_amount)
{
	amount_collected += cash_amount;
}

// Item member functions and operators

Boolean Item::vend()
{
	Boolean success;

	if (!this->is_vendable())
		return false;
	success = a_dispensing_holder->dispense(*this);
	if (!success)
		return false;
	cash_device->make_change(price);
	return true;
}

Boolean Item::operator== (Item& an_item)
{
	if ((name == an_item.get_name()) &&
	    (price == an_item.get_price()))
		return true;
	else
		return false;
}

ostream& operator<< (ostream& os, Item& an_item)
{
	String& item_name = an_item.name;
	os << item_name << " price:  "
		<< an_item.get_price();
	return os;
}

Boolean Item::is_vendable()
{
	return cash_device->got_enough(price);
}

// Item demo functions

void Item::add_item(char* item_name, float item_price,
		     Dispensing_Holder& item_holder, Cash_Device& device)
{
	Item* an_item = new Item(item_name, item_price);
	item_holder.connect_to_item(*an_item);
	an_item->connect_to_dispensing_holder(item_holder);
	an_item->connect_to_cash_device(device);
}

void Item::machine_demo()
{
	Dispensing_Holder dispensing_holder;

	Cash_Device cash_device;
	cash_device.set_amount_collected(1.50);

	this->add_item("Coca Cola", 0.75, dispensing_holder, cash_device);
	this->add_item("Pepsi", 0.75, dispensing_holder, cash_device);
	this->add_item("Jolt", 0.75, dispensing_holder, cash_device);

	dispensing_holder.vend_all(3);

}


// Dispensing_Holder member functions and operators

Boolean Dispensing_Holder::connect_to_item(Item& an_item)
{
	if (item_index < MAX_ITEMS_SIZE)
	   {
		this->items[item_index++] = &an_item;
		return true;
	   }
	return false;
}

Boolean Dispensing_Holder::disconnect_with_item(Item& an_item)
{
	for (int i = 0; i < item_index; i++)
		if (*(this->items[i]) == an_item)
		   {
			for (int j = i; j < (item_index-1); j++)
				this->items[j] = this->items[j+1];
			item_index--;
			return true;
		   }
	return false;
}

Boolean Dispensing_Holder::is_empty()
{
	if (item_index == 0)
		return true;
	else
		return false;
}


Boolean Dispensing_Holder::item_is_dispensable(Item&
	   item_to_dispense)
{
	if (*(items[0]) == item_to_dispense)
		return true;
	else
		return false;
}

void Dispensing_Holder::activate_dispenser()
{
	// no-op for simulation purposes
}

void Dispensing_Holder::print_item(Item& item_to_dispense)
{
	cout << endl << "Releasing item:  "
		<< item_to_dispense << endl;
}

Boolean Dispensing_Holder::dispense(Item& item_to_dispense)
{
	if (this->is_empty())
		return false;
	if (!this->item_is_dispensable(item_to_dispense))
		return false;

	this->activate_dispenser();
	this->print_item(item_to_dispense);
	this->disconnect_with_item(item_to_dispense);
	return true;
}

// Dispensing_Holder demo functions

void Dispensing_Holder::vend_all(int item_quantity)
{
	for (int i = 0; i < item_quantity; i++)
		    (this->items[0])->vend();
}

// Dispensing_Holder view access functions

float Dispensing_Holder:: dispensable_item_price()
{
	if (this->is_empty())
		return 0.0;
	else
		return ((items[0])->get_price());
}

String Dispensing_Holder::dispensable_item_name()
{
	if (this->is_empty())
		return String("Empty");
	else
		return ((items[0])->get_name());
}

Boolean Dispensing_Holder::vend_the_dispensable_item()
{
	Boolean success;

	if (this->is_empty())
		return false;
	success = ((items[0])->vend());
	if (!success)
		return false;
	else
		return true;
}

