/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* VMCHNCTR.CPP                                              */
/*************************************************************/

#ifndef VMCHNCTR_H
#include "vmchnctr.h"
#endif

// Member functions for Vending_Machine_View_Container

void Vending_Machine_View_Container::make_item_menu
		(Dispensing_Holder** dispensing_holders,
		int number_of_holders)
{
	menu = new Item_Console_Menu();
	menu->make_items
		(dispensing_holders, number_of_holders);
	menu->add_terminator("Quit");
	menu->add_prompt("Select an item:  ");
}

void Vending_Machine_View_Container
	::make_item_view_container(Dispensing_Holder&
	dispensing_holder)
{
	Item_View_Container* item_view_container;

	item_view_container = new Item_View_Container();
	item_view_container->build_views(dispensing_holder);
	item_view_containers[++component_count] =
			item_view_container;
}

void Vending_Machine_View_Container
	::make_cash_device_view_container(Cash_Device& a_cash_device)
{
	a_cash_device_view_container =
			new Cash_Device_View_Container();
	a_cash_device_view_container->build_views(a_cash_device);
}

void Vending_Machine_View_Container
	::build_view_containers
	(Dispensing_Holder** dispensing_holders,
	Cash_Device& a_cash_device, int number_of_holders)
{
	this->make_cash_device_view_container(a_cash_device);
	this->make_item_menu(dispensing_holders,
				number_of_holders);

	for(int i= 0; i < number_of_holders; i++)
		this->make_item_view_container
			(*dispensing_holders[i]);
}

// Display Member functions

void Vending_Machine_View_Container::
	display_item_view_containers()
{
	// I'm empty! I won't make any item view containers.
	if (component_count == -1)
		return;
	cout << endl;
	for(int i = 0 ; i<= component_count; i++)
		(item_view_containers[i])->display_components();
}

void Vending_Machine_View_Container::
	display_components()
{
	this->display_item_view_containers();
	while (menu->get_user_selection())
		{
		if (!menu->sold_out_selection())
			{
			a_cash_device_view_container->
				display_components();
			menu->execute_selection();
			}
		this->display_item_view_containers();
		}
}

//Vending_Machine_View_Container Demo functions


void Vending_Machine_View_Container::vending_machine_demo
	(Dispensing_Holder** dispensing_holders,
	Cash_Device& a_cash_device,
	int number_of_holders)
{
	this->build_view_containers(dispensing_holders,
		a_cash_device, number_of_holders);
	this->display_components();
}


void Vending_Machine_View_Container::vending_machine_demo()
{
	// Create a cash device.
	Cash_Device a_cash_device;

	// Create an array for dispensing holders.
	Dispensing_Holder** dispensing_holders =
		new Dispensing_Holder*[MAX_CONTAINER_SIZE];

	Item an_item;
	int number_of_holders;

	// Get the dispensing holders from an item.
	// I need to know how many, please.
	number_of_holders = an_item.machine_demo
		(dispensing_holders, a_cash_device);

	// Build my human interface for these
	// dispensing holders and start it running.
	this->vending_machine_demo(dispensing_holders, a_cash_device,
					number_of_holders);

	// I'm all done. I'll clean up after myself.
	delete[] dispensing_holders;
}


