/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* CON_VIWS.H                                                */
/*************************************************************/

#ifndef CC_STRING
#include "string.h"
#endif

#ifndef CC_BOOLE_H
#include "cc_boole.h"
#endif

#include <strstrea.h>
#ifndef CON_VIWS_H
#define CON_VIWS_H

const MAX_CONSOLE_MENU_SIZE = 10;

class Console_Menu {
public:

	// Constructors/destructors
	Console_Menu() { menu_size = 0; }
	~Console_Menu() { }

	// Implementors
	void add_item(String new_item);
	void add_prompt(String menu_prompt);
	void add_terminator(String quit_label);

	void display_menu();
	int get_user_selection();

	// Accessors
	int get_selection() { return selection; }

protected:

	Boolean validate_selection(int selection);

	// Data members
	String menu_items[MAX_CONSOLE_MENU_SIZE];
	String prompt_string;
	int menu_size;
	int selection;
};

template<class Object_Type, class Value_Function_Type>
class Console_Display_Box {
public:

	// Constructors/destructors
	Console_Display_Box() { label_string = ""; }
	~Console_Display_Box() { }

	// Accessors
	String get_label_string() { return label_string; }
	void set_label_string(String a_string)
		{ label_string = a_string; }

	Object_Type& get_object() { return *object; }
	void set_object (Object_Type& an_object)
		{ object = &an_object; }

	Value_Function_Type get_value_function()
		{ return value_function; }
	void set_value_function(Value_Function_Type a_function)
		{ value_function = a_function;}

	void display_label_string();
	void display_value();

private:

	// Data members
	String label_string;
	Object_Type* object;
	Value_Function_Type value_function;
};

template<class Object_Type, class Value_Function_Type>
void Console_Display_Box<Object_Type, Value_Function_Type>
	::display_value()
{
	this->display_label_string();
	cout << (object->*value_function)();
}

template<class Object_Type, class Value_Function_Type>
void Console_Display_Box<Object_Type, Value_Function_Type>
		::display_label_string()
{
	if (label_string == String(""))
		return;
	else
		cout << label_string << " ";
}

#endif