/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* DATE.CPP                                                  */
/*************************************************************/

#include <iostream.h>
#include <ctype.h>
#ifndef DATE_H
#include "date.h"
#endif


long Date::seconds_per_day = 86400;

Date::Date()
{
	calendar_time = time(NULL);
}

Date::Date(int day, int month, int year)
{
	this->set_date(day, month, year);
}

Weekday Date::day_of_week()
{
	struct tm* time_struct = localtime(&calendar_time);
	return (Weekday)time_struct->tm_wday;
}

int Date::day_of_month()
{
	struct tm* time_struct = localtime(&calendar_time);
	return time_struct->tm_mday;
}

int Date::year()
{
	struct tm* time_struct = localtime(&calendar_time);
	return time_struct->tm_year;
}

int Date::month()
{
// Months range from 0 (January) to 11 (December)
	struct tm* time_struct = localtime(&calendar_time);
	return time_struct->tm_mon + 1;
}

void Date::set_date(int day, int month, int year)
{
	struct tm time_struct = {0};
	time_t temp_time;
	time_struct.tm_mday = day;
// Months range from 0 (January) to 11 (December)
	time_struct.tm_mon = month - 1;
	time_struct.tm_year = year;
	temp_time = mktime(&time_struct);
	if (temp_time != (time_t)-1)
		calendar_time = temp_time;
}

char* Date::print_string()
{
	char* date_string = new char[30];
	struct tm* time_struct = localtime(&calendar_time);

	strftime(date_string, 30, "%x", time_struct);

	return date_string;
}

ostream& operator<< (ostream& os, Date& aDate)
{
	char* date_string = aDate.print_string();
	os << date_string;
	return os;
}

Date Date::operator++()
{
	calendar_time += seconds_per_day;
	return *this;
}

Date Date::operator++(int)
{
	calendar_time += seconds_per_day;
	return *this;
}

Date Date::operator--()
{
	calendar_time -= seconds_per_day;
	return *this;
}

Date Date::operator--(int)
{
	calendar_time -= seconds_per_day;
	return *this;
}

int operator< (Date& date1, Date& date2)
{
	return date1.calendar_time < date2.calendar_time;
}

int operator> (Date& date1, Date& date2)
{
	return date1.calendar_time > date2.calendar_time;
}

int operator== (Date& date1, Date& date2)
{
	return date1.calendar_time == date2.calendar_time;
}
