/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* FORMATS.H                                                 */
/*************************************************************/

#include <fstream.h>
#include <iostream.h>
#include <strstrea.h>
#include "time.h"
#include "date.h"
#include "cc_boole.h"
#include "olist.h"
#include "string.h"

#ifndef STORAGE2_H
#include "storage2.h"
#endif

#ifndef SALES_H
#include "sales.h"
#endif

#ifndef FORMATS_H
#define FORMATS_H

template<class Transaction_Type>
class Transaction_Format : public
	PDC_Object_Format<Transaction_Type> {
public:
	// Constructors/destructors
	Transaction_Format() { }
	~Transaction_Format() { }

	// Implementors
	void store_class_name(ofstream& a_stream);
	void store_attribute_values(ofstream& a_stream);
	void store_connections(ofstream& a_stream) ;
	void restore_connections(ifstream& a_stream,
		Object_Table* an_object_table);
	void restore_attribute_values(ifstream& a_stream);

private:

};

template<class Transaction_Type>
void
Transaction_Format<Transaction_Type>::store_class_name
	(ofstream& a_stream)
{
	this->store_tag_value(a_stream, "className", "Transaction");
}

template<class Transaction_Type>
void Transaction_Format<Transaction_Type>::
	store_attribute_values(ofstream& a_stream)
{
	Date& a_date = pdc_object->get_date();
	Time& a_time = pdc_object->get_time();

	this->store_date(a_stream, a_date);
	this->store_time(a_stream, a_time);
}

template<class Transaction_Type>
void Transaction_Format<Transaction_Type>::
	store_connections(ofstream& a_stream)
{
	Line_Item_List& line_items =
		pdc_object->get_line_items();
	Line_Item* item;
	int line_item_ID;
	while (!line_items.end_of_list())
		{
		item = line_items.current();
		line_item_ID = item->get_object_ID();
		this->store_tag(a_stream, "lineItem");
		this->store_ID(a_stream, line_item_ID);
		line_items++;
		}
}

template<class Transaction_Type>
void Transaction_Format<Transaction_Type>::
	restore_attribute_values(ifstream& a_stream)
{
	Date& a_date = this->restore_date(a_stream);
	Time& a_time = this->restore_time(a_stream);

	pdc_object->set_date(a_date);
	pdc_object->set_time(a_time);
}

template<class Transaction_Type>
void Transaction_Format<Transaction_Type>::restore_connections(ifstream&
	a_stream, Object_Table* an_object_table)
{
	int object_ID;
	Line_Item* line_item;

	while ((this->restore_tag(a_stream, "lineItem")) != 0)
		{
		object_ID = this->restore_ID(a_stream);
		line_item =
			(Line_Item*)an_object_table->
				object_at_ID(object_ID);
		pdc_object->connect_to_line_item(*line_item);
		}
}


class Sales_Transaction_Format :
	public Transaction_Format<Sales_Transaction> {
public:

	// Constructors/destructors
	Sales_Transaction_Format() { }
	~Sales_Transaction_Format() { }

	// Implementors
	void store_class_name(ofstream& a_stream);
	void store_attribute_values(ofstream& a_stream);

	void restore_attribute_values(ifstream& a_stream);

private:

};

class Customer_Format : public PDC_Object_Format<Customer> {
public:

	// Constructors/destructors
	Customer_Format() { }
	~Customer_Format() { }

	// Implementors
	void store_class_name(ofstream& a_stream);
	void store_attribute_values(ofstream& a_stream);
	void store_connections(ofstream& a_stream);

	void restore_connections(ifstream& a_stream,
				Object_Table* object_table);
	void restore_attribute_values(ifstream& a_stream);

private:

};




class Line_Item_Format : public PDC_Object_Format<Line_Item> {
public:

	// Constructors/destructors
	Line_Item_Format() { }
	~Line_Item_Format() { }

	// Implementors
	void store_class_name(ofstream& a_stream);
	void store_attribute_values(ofstream& a_stream);
	void store_connections(ofstream& a_stream) ;

	void restore_connections(ifstream& a_stream,
				Object_Table* object_table);
	void restore_attribute_values(ifstream& a_stream);

private:

};


class Product_Format : public PDC_Object_Format<Product> {
public:

	// Constructors/destructors
	Product_Format() { }
	~Product_Format() { }

	// Implementors
	void store_class_name(ofstream& a_stream);
	void store_attribute_values(ofstream& a_stream);

	void restore_connections(ifstream& a_stream,
				Object_Table* object_table) {}

	void restore_attribute_values(ifstream& a_stream);

private:

};

class Sales_Domain_Class_Map : public Class_Map {
public:

	// Constructors/destructors
	Sales_Domain_Class_Map();
	~Sales_Domain_Class_Map() { }

	// Implementors
	int table_class_code() {return OBJECT_TABLE;}
	void restore_object(int object_ID, int class_code,
		 Object_Table* an_object_table);
};

#endif



