/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* OLIST.H                                                   */
/*************************************************************/

#include<stdlib.h>
#ifndef CC_BOOLEAN_H
#include "cc_boole.h"
#endif

#ifndef OLIST_H
#define OLIST_H

// enum Boolean {false, true}

const NULL_ELEMENT = 0;
const NULL_ITEM = 0;

// Forward reference
template<class Item_Type>class Ordered_List;

// Ordered_List_Element Class Declaration

template<class Item_Type>
class Ordered_List_Element {
public:
	Ordered_List_Element () { next_element = NULL_ELEMENT; }
	Ordered_List_Element (const Item_Type& new_item)
				{ next_element = NULL_ELEMENT;
					item = new_item;}
	~Ordered_List_Element () { }

	friend class Ordered_List<Item_Type>;

private:

	// Data members
	Ordered_List_Element<Item_Type>* next_element;
	Item_Type item;
};

// Ordered_List Class Declaration

template<class Item_Type>
class Ordered_List {
public:
	Ordered_List() { first_element = NULL_ELEMENT;
			 current_element = NULL_ELEMENT;
			 last_element = NULL_ELEMENT;
			 number_of_items = 0; }
	~Ordered_List() { this->delete_items(); }


	void add(const Item_Type& new_item);
	Boolean remove(const Item_Type& an_item);
	void reset() { current_element = first_element; }

	Item_Type current();
	Item_Type item_at(int location);

	Boolean is_empty();
	Boolean includes(const Item_Type& an_item);
	Boolean end_of_list();

	int size() { return number_of_items; }

	Item_Type operator++ ();	// Prefix increment
	Item_Type operator++ (int);	// Postfix increment

	friend ostream& operator<<(ostream& os,
			Ordered_List<Item_Type>&);

private:

	// Utility functions
	void delete_items();
	void remove_after(Ordered_List_Element<Item_Type>*
				previous_element);
	Ordered_List_Element<Item_Type>* locate(const Item_Type& an_item);

	// Data members
	Ordered_List_Element<Item_Type>* first_element;
	Ordered_List_Element<Item_Type>* current_element;
	Ordered_List_Element<Item_Type>* last_element;

	int number_of_items;
};

// Ordered_List Member Functixons

// Public: Add/Remove
template<class Item_Type>
void Ordered_List<Item_Type>::add(const Item_Type& new_item)
{
	Ordered_List_Element<Item_Type>* new_element =
		new Ordered_List_Element<Item_Type>(new_item);

	if (first_element == NULL_ELEMENT)
		{
		first_element = new_element;
		current_element = new_element;
		last_element = new_element;
		}
	else
		{
		last_element->next_element = new_element;
		last_element = new_element;
		}
	number_of_items++;
}


template<class Item_Type>
Boolean Ordered_List<Item_Type>::remove(const Item_Type& an_item)
{
	Ordered_List_Element<Item_Type>* previous_element;

	Boolean found;
	previous_element = this->locate(an_item);
	if (previous_element == NULL_ELEMENT)
		return found.be_false();
	if (previous_element == first_element)
		this->remove_after(NULL_ELEMENT);
	else
		this->remove_after(previous_element);
	this->reset();
	number_of_items--;
	return found.be_true();
}

// Public: Iteration

template<class Item_Type>
Item_Type Ordered_List<Item_Type>::current()
{
	if (current_element == NULL_ELEMENT)
		{
		cerr << "Attempt to read from empty list\n";
		exit(-1);
		}
	return current_element->item;
}

template<class Item_Type>
Item_Type Ordered_List<Item_Type>::item_at(int location)
{
	if (this->is_empty())
		{
		cerr << "Attempted: to access item in empty list.\n";
		exit(-1);
		}
	if ((location >= number_of_items) || (location < 0 ))
		{
		cerr << "Subscript out of bounds for list.\n";
		exit(-1);
		}
	this->reset();
	for(int i = 0; i < location; i++)
		this->operator++(0);
	return current_element->item;
}


// Public: List Tests

template<class Item_Type>
Boolean Ordered_List<Item_Type>::is_empty()
{
	return (first_element == NULL_ELEMENT);
}

template<class Item_Type>
Boolean Ordered_List<Item_Type>::includes(const Item_Type& an_item)
{
	Boolean success(first_element !=  NULL_ELEMENT);
	if (!success)
		return success;
	success = (this->locate(an_item) != NULL_ELEMENT);
	return success;
}

template<class Item_Type>
Boolean Ordered_List<Item_Type>::end_of_list()
{
	return Boolean(current_element == NULL_ELEMENT);
}


template<class Item_Type>
Item_Type Ordered_List<Item_Type>::operator++()
{
	if (current_element == NULL_ELEMENT)
		{
		cerr << "Attempted: operator++() past end of list.\n";
		exit(-1);
		}

	Ordered_List_Element<Item_Type>* temp_element;
	temp_element = current_element;
	current_element = current_element->next_element;

	return temp_element->item;
}

template<class Item_Type>
Item_Type Ordered_List<Item_Type>::operator++(int)
{
	if (current_element == NULL_ELEMENT)
		{
		cerr << "Attempted: operator++(int) past end of list.\n";
		exit(-1);
		}

	current_element = current_element->next_element;
	return current_element->item;
}


template<class Item_Type>
ostream& operator<<(ostream& os,
		Ordered_List<Item_Type>& an_ordered_list)
{
	while(!an_ordered_list.end_of_list())
		{
		cout << an_ordered_list.current() << endl;
		an_ordered_list++;
		}
	return os;
}


// Private: Utility functions
template<class Item_Type>
void Ordered_List<Item_Type>::delete_items()
{
	if (first_element == NULL_ELEMENT)
		return;

	Ordered_List_Element<Item_Type>* temp_element;
	current_element = first_element->next_element;
	delete first_element;
	while (current_element != NULL_ELEMENT)
		{
		temp_element = current_element;
		current_element = current_element->next_element;
		delete temp_element;
		}
}

template<class Item_Type>
void Ordered_List<Item_Type>::remove_after(
		Ordered_List_Element<Item_Type>* previous_element)
{
	Ordered_List_Element<Item_Type>* temp_element;

	if (previous_element == NULL_ELEMENT)
		{
		temp_element = first_element;
		first_element = first_element->next_element;
		delete temp_element;
		}
	else
		{
		temp_element = previous_element->next_element;
		previous_element->next_element =
				temp_element->next_element;
		delete temp_element;
		}
}

template<class Item_Type>
Ordered_List_Element<Item_Type>*
	Ordered_List<Item_Type>::locate(const Item_Type& an_item)
{
	Ordered_List_Element<Item_Type>* previous_element;
	Boolean found;

	this->reset();
	previous_element = current_element;
	while(!this->end_of_list())
		{
		if (current_element->item == an_item)
			{
			found.be_true();
			break;
			}
		previous_element = current_element;
		this->operator++(0);  // Call postfix++
		}
	if (!found)
		return NULL_ELEMENT;
	else
		return previous_element;
}


#endif