/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* SALES1.CPP                                                */
/*************************************************************/

#include <stdio.h>
#include <stdlib.h>
#ifndef SALES1_H
#include "sales1.h"
#endif


// Product member functions and operators

Product::Product()
{
	name = "Widget";
	price = 2.5;
}

Product::Product(String a_name, float a_price)
{
	name = a_name;
	price = a_price;
}

ostream& operator<< (ostream& os, Product& a_product)
{
	os.width(12);			// Field width of 12
					// Left justified
	os.setf(ios::left, ios::adjustfield);
	os << a_product.name;
	os.width(0);			// Restore default.
					// Restore default.
	os.setf(ios::right, ios::adjustfield);
	os << " price: ";
	os.precision(2);		// Two decimal places.
	os.width(5);
	os.setf(ios::showpoint);        // Show trailing zeros.
	os << a_product.price << " ";
	os.setf(0, ios::floatfield);    // Restore default.
	os.precision(6);		// Restore default.
	return os;
}

Boolean Product::operator==(Product& a_product)
{
	Boolean same_name(name == a_product.get_name());
	Boolean same_price(price == a_product.get_price());

	return (same_name && same_price);
}



// Line_Item member functions and operators

Line_Item::Line_Item(Product& a_product, int
		product_quantity)
{
	product = &a_product;
	quantity = product_quantity;
	line_number = 0;
}

void Line_Item::connect_to_product(Product& a_product)
{
	product = &a_product;
}


ostream& operator<< (ostream& os, Line_Item& a_line_item)
{
	os << a_line_item.line_number
		<< ". "
		<< *a_line_item.product
		<< " quantity: "
		<< a_line_item.quantity << " "
		<< " total: " ;
	os.precision(2);
	os.width(5);
	os.setf(ios::showpoint);
	os	<< a_line_item.calculate_total()
		<< endl;
	os.setf(0, ios::floatfield);    // Restore default.
	os.precision(6);		// Restore default.
	return os;
}


Boolean Line_Item::operator== (Line_Item& a_line_item)
{
	return(this->line_number ==
			a_line_item.get_line_number());
}

float Line_Item::calculate_total()
{
	return (this->quantity * product->get_price());
}

