/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* SALES2.CPP                                                */
/*************************************************************/

#include <stdio.h>
#include <stdlib.h>
#ifndef SALES2_H
#include "sales2.h"
#endif


// Product member functions and operators

Product::Product()
{
	name = "Widget";
	price = 2.5;
}

Product::Product(String a_name, float a_price)
{
	name = a_name;
	price = a_price;
}

ostream& operator<< (ostream& os, Product& a_product)
{
	os.width(12);			// Field width of 12
					// Left justified
	os.setf(ios::left, ios::adjustfield);
	os << a_product.name;
	os.width(0);			// Restore default.
					// Restore default.
	os.setf(ios::right, ios::adjustfield);
	os << " price: ";
	os.precision(2);		// Two decimal places.
	os.width(5);
	os.setf(ios::showpoint);        // Show trailing zeros.
	os << a_product.price << " ";
	os.setf(0, ios::floatfield);    // Restore default.
	os.precision(6);		// Restore default.
	return os;
}

Boolean Product::operator==(Product& a_product)
{
	Boolean same_name(name == a_product.get_name());
	Boolean same_price(price == a_product.get_price());

	return (same_name && same_price);
}



// Line_Item member functions and operators

Line_Item::Line_Item(Product& a_product, int
		product_quantity)
{
	product = &a_product;
	quantity = product_quantity;
	line_number = 0;
}

void Line_Item::connect_to_product(Product& a_product)
{
	product = &a_product;
}


ostream& operator<< (ostream& os, Line_Item& a_line_item)
{
	os << a_line_item.line_number
		<< ". "
		<< *a_line_item.product
		<< " quantity: "
		<< a_line_item.quantity << " "
		<< " total: " ;
	os.precision(2);
	os.width(5);
	os.setf(ios::showpoint);
	os	<< a_line_item.calculate_total()
		<< endl;
	os.setf(0, ios::floatfield);    // Restore default.
	os.precision(6);		// Restore default.
	return os;
}


Boolean Line_Item::operator== (Line_Item& a_line_item)
{
	return(this->line_number ==
			a_line_item.get_line_number());
}

float Line_Item::calculate_total()
{
	return (this->quantity * product->get_price());
}


// Transaction member functions and operators

void Transaction::connect_to_line_item(Line_Item& a_line_item)
{
	line_items.add(&a_line_item);
	a_line_item.set_line_number(line_items.size());
}

void Transaction::disconnect_with_line_item
			(Line_Item& a_line_item)
{
	line_items.remove(&a_line_item);
	this->renumber_line_items();
}

void Transaction::renumber_line_items()
{
	int number_of_items = line_items.size();
	Line_Item* a_line_item;
	for (int i = 0; i < number_of_items; i ++)
		{
		a_line_item = line_items.item_at(i);
		a_line_item->set_line_number(i+ 1);
		}
}

ostream& operator<< (ostream& os, Transaction&
	a_transaction)
{
	os << a_transaction.date << " "
	   << a_transaction.time << endl;
	a_transaction.line_items.reset();
	while(!a_transaction.line_items.end_of_list())
		{
		os << *a_transaction.line_items.current();
		a_transaction.line_items++;
		}
	os << " Total cost: "
	   <<  a_transaction.calculate_total()
	   << endl;
	os.setf(0, ios::floatfield);
	os.precision(6);
	return os;
}


Boolean Transaction::operator== (Transaction& a_transaction)
{
	return ((date == a_transaction.get_date()) &&
		(time == a_transaction.get_time()));
}

// Sales_Transaction member functions and operators


ostream& operator<< (ostream& os, Sales_Transaction& a_transaction)
{
	os << endl << a_transaction.date << " "
	   << a_transaction.time << endl;
	a_transaction.line_items.reset();
	while(!a_transaction.line_items.end_of_list())
		{
		os << *a_transaction.line_items.current();
		a_transaction.line_items++;
		}
	os.precision(2);
	os.width(5);
	os.setf(ios::showpoint);
	os << "Total Cost: "
	   << a_transaction.calculate_total()
	   << endl;
	os.setf(0, ios::floatfield);
	os.precision(6);
	os << "Method of Payment:  "
	   << a_transaction.method_of_payment
	   << endl;
	return os;
}



float Sales_Transaction::calculate_total()
{
	float subtotal = 0.0;
	Line_Item* a_line_item;

	line_items.reset();
	while(!line_items.end_of_list())
		{
		a_line_item = line_items.current();
		subtotal = subtotal + a_line_item->calculate_total();
		line_items++;
		}
	return subtotal;
}

Boolean Sales_Transaction::operator== (Sales_Transaction& a_sales_transaction)
{
	Boolean same_base = Transaction::operator==(a_sales_transaction);
	return (same_base &&
		(method_of_payment ==
			 a_sales_transaction.get_method_of_payment()));
}

