/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* SALES6.H                                                  */
/*************************************************************/

#include <iostream.h>
#include <strstrea.h>
#include "time.h"
#include "date.h"
#include "cc_boole.h"
#include "olist.h"
#include "string.h"


#ifndef SALES6_H
#define SALES6_H

// Forward references
class Sales_Transaction;
class Line_Item;
class Customer;
class Product;

typedef Ordered_List<Sales_Transaction*>Transaction_List;
typedef Ordered_List<Line_Item*>Line_Item_List;


class Product {
public:

	// Constructors/destructors
	Product();
	Product(String a_name, float a_price);
	~Product() { }

	// Accessors
	String get_name() { return name; }
	void set_name(String new_value) { name = new_value; }

	float get_price() { return price; }
	void set_price(float new_value) { price = new_value; }

	friend ostream& operator<<(ostream& os, Product& a_product);
	Boolean operator==(Product& a_product);

private:

	// Data members
	String name;
	float price;
};


class Line_Item {
public:

	// Constructors/destructors
	Line_Item() { quantity = 0; line_number = 0;}
	Line_Item(Product& a_product, int product_quantity);
	~Line_Item() { }

	// Implementors
	float calculate_total();

	// Accessors
	int get_quantity() { return quantity; }
	void set_quantity(int new_value) { quantity =
		new_value; }

	int get_line_number() { return line_number; }
	void set_line_number(int new_value)
		{ line_number = new_value; }

	Product& get_product() { return *product; }
	void connect_to_product(Product& a_product);

	// Overloaded operators for Line_Items
	Boolean operator== (Line_Item& a_transaction_line_item);
	friend ostream& operator<< (ostream& os, Line_Item&
						 a_transaction_line_item);

	// Demo functions
	Line_Item* for_product_mega_widget(int quantity);
	Line_Item* for_product_super_widget(int quantity);
	Line_Item* for_product_bowling_shoes(int quantity);

	// Member functions to support human interaction
	String quantity_string() { return String(quantity); }
	String total_string() { return
		String(this->calculate_total()); }

private:

	// Data members
	Product* product;
	int line_number;
	int quantity;
};


class Transaction {
public:

	// Constructors/destructors
	Transaction() { }
	~Transaction() { };

	// Implementors
	virtual float calculate_total() = 0;

	// Accessors
	Date get_date() { return date; }
	Time get_time() { return time; }
	void set_date(Date new_value)  { date = new_value; }
	void set_time(Time new_value)  { time = new_value; }

	Customer& get_customer() { return *customer; }
	void connect_to_customer(Customer& a_customer)
		{ customer = &a_customer; }

	void connect_to_line_item(Line_Item& a_line_item);
	void disconnect_with_line_item(Line_Item& a_line_item);
	Line_Item_List& get_line_items() {line_items.reset();
		return line_items; }

	// Overloaded operators for Transactions
	Boolean operator== (Transaction& a_transaction);
	friend ostream& operator<< (ostream& os, Transaction& a_transaction);

	// Member functions to support human interaction
	String get_date_time();
	String total_string() { return
		String(this->calculate_total()); }

protected:

	// Helper member functions
	void renumber_line_items();

	// Data members
	Date date;
	Time time;
	Customer* customer;
	Line_Item_List line_items;

};

class Sales_Transaction : public Transaction {
public:

	// Constructors/destructors
	Sales_Transaction() { method_of_payment = "cash";}
	~Sales_Transaction() { }

	// Implementors
	float calculate_total();

	// Accessors
	String get_method_of_payment() { return
		method_of_payment; }
	void paid_with_cash() { method_of_payment = "cash"; }
	void sold_on_credit() { method_of_payment = "credit"; }

	// Overloaded operators for Sales_Transactions
	friend ostream& operator<< (ostream& os, Sales_Transaction&
						 a_sales_transaction);

	Boolean operator== (Sales_Transaction& a_sales_transaction);

private:

	// Data members
	String method_of_payment;
};

class Customer {
public:

	// Constructors/destructors
	Customer();
	Customer(String a_name, String an_address);
	~Customer() { }

	// Implementors

	// Accessors
	String get_name() { return name; }
	void set_name(String new_value) { name = new_value; }

	String get_address() { return address; }
	void set_address(String new_value) { address =
		new_value; }

	Transaction_List& get_transactions();
	void connect_to_transaction(Sales_Transaction&
		a_transaction);
	void disconnect_with_transaction
		(Sales_Transaction& a_transaction);

	// Overloaded operators for Customer
	Boolean operator== (Customer& a_customer);
	friend ostream& operator<< (ostream& os, Customer& a_customer);

	// Demo functions
	Customer& customer_demo();

private:

	// Data members
	String name;
	String address;
	Transaction_List transactions;

};


#endif