/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* STORAGE2.CPP                                              */
/*************************************************************/

#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>

#include "string.h"

#ifndef STORAGE2_H
#include "storage2.h"
#endif

// initialize PDC_Object next_object_ID
int PDC_Object::next_object_ID = 0;

// Member functions for PDC_Object

void PDC_Object::register_in_table(Object_Table*
	an_object_table)
{
	an_object_table->register_object(object_ID, this);
	this->register_connections(an_object_table);
}

void PDC_Object::store_using_table(ofstream& a_stream,
	Object_Table* an_object_table)
{
	int class_code = this->get_class_code();
	Class_Map* class_map = an_object_table->get_class_map();
	Object_Tag_Format* format =
		class_map->get_format_for(class_code);
	format->set_object(this);

	format->store(a_stream);
}

void PDC_Object::restore_using_table(ifstream& a_stream,
	Object_Table* an_object_table)
{
	int class_code = this->get_class_code();
	Class_Map* class_map = an_object_table->get_class_map();
	Object_Tag_Format* format =
		class_map->get_format_for(class_code);
	format->set_object(this);

	format->restore(a_stream, an_object_table);
}

// Member functions for Object_Tag_Format

void inline Object_Tag_Format::store_tag(ofstream&
	a_stream, String tag_name)
{
	a_stream << "<" << tag_name << ">";
}

void inline Object_Tag_Format::store_value(ofstream&
	a_stream, String tag_value)
{
	a_stream << tag_value << endl;
}

void inline Object_Tag_Format::store_tag_value(ofstream&
	a_stream, String tag_name, String tag_value)
{
	this->store_tag(a_stream, tag_name);
	this->store_value(a_stream, tag_value);
}

void inline Object_Tag_Format::store_defn_start
	(ofstream& a_stream)
{
	this->store_tag(a_stream, "ObjectDefn");
	a_stream << endl;
}

void inline Object_Tag_Format::store_defn_end(ofstream&
	a_stream)
{
	// Use two backslahes. The first backslash indicates a
	// special character follows. This allows the second
	// backslash to be printed.
	this->store_tag(a_stream, "\\ObjectDefn");
	a_stream << endl;
}

void Object_Tag_Format::store_ID(ofstream& a_stream,
	int object_ID)
{
	// Create a string from the object ID.
	String id_string(object_ID);

	this->store_tag_value(a_stream, "id", id_string);
}

void Object_Tag_Format::store_date(ofstream& a_stream,
	Date& a_date)
{
	String& separator = String("/");
	String& date_string = String(a_date.month())
		+ separator
		+ String(a_date.day_of_month())
		+ separator
		+ String(a_date.year());

	this->store_tag_value(a_stream, "date", date_string);
}

void Object_Tag_Format::store_time(ofstream& a_stream,
	Time& a_time)
{
	this->store_tag_value(a_stream, "time",
		a_time.print_string());
}

void Object_Tag_Format::store(ofstream& a_stream)
{
	this->store_defn_start(a_stream);
	this->store_class_name(a_stream);
	this->store_attribute_values(a_stream);
	this->store_connections(a_stream);
	this->store_defn_end(a_stream);
}


// The restoring member functions

Boolean Object_Tag_Format::restore_tag(ifstream& a_stream,
	String tag_name)
{
	char ch, last_char = '>';
	char tag_read[80];

	// Current position in the stream.
	long position = a_stream.tellg();

	// Read the first character, '<'.
	a_stream.get(ch);

	// Read the tag name up to the '>'
	a_stream.get(tag_read, 80, last_char);

	// Check if tag read is the same as tag name.
	if (String(tag_read) == tag_name)
		{
		// The same. Get the '>' from the stream.
		a_stream.get(ch);
		return true;
		}
	else
		{
		// Not the same. Reset stream position.
		a_stream.seekg(position);
		return false;
		}
}

String Object_Tag_Format::restore_value(ifstream& a_stream)
{
	char ch, last_char = '<';
	char* tag_value = new char[80];

	// Read up to the start of the next tag.
	a_stream.get(tag_value, 80, last_char);

	// Drop the last character, if it's a newline character.
	int size = strlen(tag_value);
	if (tag_value[size-1] == '\n')
		tag_value[size-1] = '\0';
	return tag_value;
}

String Object_Tag_Format::restore_tag_value(ifstream&
	a_stream, String tag_name)
{
	this->restore_tag(a_stream, tag_name);
	return this->restore_value(a_stream);
}

int Object_Tag_Format::restore_ID(ifstream& a_stream)
{
	if (!(this->restore_tag(a_stream, "id")))
		return -1;
	String& id_string = this->restore_value(a_stream);
	int object_ID = id_string.as_integer();
	return object_ID;
}

Date& Object_Tag_Format::restore_date(ifstream& a_stream)
{
	char separator = '/';
	String& date_value_string =
		this->restore_tag_value(a_stream, "date");
	String& month_string = date_value_string.break_at(separator);
	String& day_string = date_value_string.break_at(separator);
	String& year_string = date_value_string;
	Date* restored_date = new Date(day_string.as_integer(),
		month_string.as_integer(),
		year_string.as_integer());
	return *restored_date;
}

Time& Object_Tag_Format::restore_time(ifstream& a_stream)
{
	char separator = ':';
	String& time_value_string =
		this->restore_tag_value(a_stream, "time");
	String& hour_string =
		time_value_string.break_at(separator);
	String& minute_string =
		time_value_string.break_at(separator);
	String& second_string = time_value_string;
	Time* restored_time = new Time(hour_string.as_integer(),
		minute_string.as_integer(),
		second_string.as_integer());
	return *restored_time;
}

void Object_Tag_Format::restore(ifstream& a_stream,
				Object_Table* object_table)
{
	this->read_defn_start(a_stream);
	this->restore_class_name(a_stream);
	this->restore_attribute_values(a_stream);
	this->restore_connections(a_stream,
		object_table);
	this->read_defn_end(a_stream);
}

void Object_Tag_Format::read_defn_start(ifstream& a_stream)
{
	char ch;
	this->restore_tag(a_stream, "ObjectDefn");
	a_stream.get(ch);
}

void Object_Tag_Format::read_defn_end(ifstream& a_stream)
{
	char ch;
	this->restore_tag(a_stream, "\\ObjectDefn");
	a_stream.get(ch);
}

String Object_Tag_Format::restore_class_name(ifstream& a_stream)
{
	this->restore_tag(a_stream, "className");
	return this->restore_value(a_stream);
}

// Member functions for Object_Table

Object_Table_Object_Iterator& Object_Table::get_objects()
{
	Object_Table_Object_Iterator* object_iterator
		= new Object_Table_Object_Iterator(*this);
	return *object_iterator;
}

Boolean Object_Table::find_ID(int object_ID)
{
	// Check if empty.
	if (table_index == -1)
		return false;
	int i = 0;
	Boolean found = false;
	while ( (i <= table_index) && (!found) )
		{
		if (object_IDs[i] == object_ID)
			found = true;	// found it!
		else
			found = false;
		i++;
		}
	return found;
}

void* Object_Table::object_at_ID(int object_ID)
{
	int i = 0;

	PDC_Object* pdc_object;
	while  ((i <= table_index) &&
		(object_IDs[i] != object_ID))
		i = i++;
	pdc_object = objects[i];
	return (void*)pdc_object;
}

void Object_Table::register_object(int object_ID,
	PDC_Object* pdc_object)
{
	if (!this->find_ID(object_ID))
		{
		// Didn't find the object_id; register the object.
		object_IDs[++table_index] = object_ID;
		objects[table_index] = pdc_object;
		}
}

void Object_Table::store(ofstream& a_stream)
{
	Object_Tag_Format* format =
		class_map->get_format_for_table();
	format->set_object(this);
	format->store(a_stream);
}

void Object_Table::store_objects(ofstream& a_stream)
{
	// Get mu object iterator.
	Object_Table_Object_Iterator next_object =
		this->get_objects();
	PDC_Object* pdc_object;

	while ((pdc_object = next_object()) != 0)
		pdc_object->store_using_table(a_stream, this);
}

Boolean Object_Table::save_to_file(char* filename)
{
	//Open the file and get its output file stream.
	ofstream a_stream(filename, ios::out);

	// Ensure file has been opened successfully.
	if (!a_stream)
		{
		cout << "Cannot open " << filename << "for output"
			<< endl;
		return false;
		}

	this->store(a_stream);
	this->store_objects(a_stream);
	a_stream.close();
	return true;
}

void Object_Table::restore(ifstream& a_stream)
{
	Object_Tag_Format* format =
		class_map->get_format_for_table();
	format->set_object(this);
	format->restore(a_stream, this);
}


void Object_Table::restore_objects(ifstream& a_stream)
{
	Object_Table_Object_Iterator next_object =
		this->get_objects();
	PDC_Object* pdc_object;
	while ((pdc_object = next_object()) != 0)
		pdc_object->restore_using_table(a_stream, this);
}

Boolean Object_Table::read_from_file(char* filename)
{
	//Open the file and get its output file stream.
	ifstream a_stream(filename);

	// Ensure file has been opened successfully.
	if (!a_stream)
		{
		cout << "Cannot open " << filename << "for input"
			<< endl;
		return false;
		}
	this->restore(a_stream);
	this->restore_objects(a_stream);
	a_stream.close();
	return true;
}

// Object Table Format Member Functions

void Object_Table_Format::store_class_name(ofstream& a_stream)
{
	this->store_tag_value(a_stream, "className",
		"Object_Table");
}

void Object_Table_Format::store_class_code
	(ofstream& a_stream, int class_code)
{
	String code_string(class_code);
	this->store_tag_value(a_stream, "classCode",
		code_string);
}

void Object_Table_Format::store_table_start(ofstream&
	a_stream)
{
	this->store_tag(a_stream, "ObjectTable");
	a_stream << endl;
}

void Object_Table_Format::store_table_end(ofstream&
	a_stream)
{
	this->store_tag(a_stream, "\\ObjectTable");
	a_stream << endl;
}


void Object_Table_Format::store_object_IDs(ofstream&
	a_stream)
{
	PDC_Object* pdc_object;

	// Get the iterator for the objet table's PDC objects.
	Object_Table_Object_Iterator& next_object
		= an_object_table->get_objects();

	int class_code;

	while ((pdc_object = next_object()) != 0)
		{
		// Store the pdc object's class code.
		class_code = pdc_object->get_class_code();
		this->store_class_code(a_stream, class_code);
		this->store_tag(a_stream, "objectID");
		int id = pdc_object->get_object_ID();
		this->store_ID(a_stream, id);
		}
}

void Object_Table_Format::store(ofstream& a_stream)
{
	this->store_defn_start(a_stream);
	this->store_class_name(a_stream);
	this->store_table_start(a_stream);
	this->store_object_IDs(a_stream);
	this->store_table_end(a_stream);
	this->store_defn_end(a_stream);
}


int Object_Table_Format::restore_class_code(ifstream&
	a_stream)
{
	if (!(this->restore_tag(a_stream, "classCode")))
		return -1;
	String& code_string = this->restore_value(a_stream);
	int class_code = code_string.as_integer();
	return class_code;
}

void Object_Table_Format::read_table_start
	(ifstream& a_stream)
{
	char ch;

	this->restore_tag(a_stream, "ObjectTable");
	a_stream.get(ch);
}

void Object_Table_Format::read_table_end(ifstream&
	a_stream)
{
	char ch;

	this->restore_tag(a_stream, "\\ObjectTable");
	a_stream.get(ch);
}

void Object_Table_Format::restore_object_IDs(ifstream&
	a_stream, Object_Table* new_object_table)
{
	an_object_table = new_object_table;

	// Get the class map for the object table.
	Class_Map* class_map = an_object_table->get_class_map();

	int class_code, object_ID;

	// Read the next class code from the stream.
	// Stop when I encounter something other than a class
	// code.
	while ((class_code = this->restore_class_code(a_stream))
		!= -1)
		{
		// Read the object identifier.
		this->restore_tag(a_stream, "objectID");
		object_ID = this->restore_ID(a_stream);

		// Ask the class map to recreate the object using
		// its class code and object identifier. The class
		// map then registers the object in the object table.
		class_map->restore_object(object_ID, class_code,
			an_object_table);
		}
}

void Object_Table_Format::restore(ifstream& a_stream,
	Object_Table* object_table)
{
	this->read_defn_start(a_stream);
	this->restore_class_name(a_stream);
	this->read_table_start(a_stream);
	this->restore_object_IDs(a_stream, object_table);
	this->read_table_end(a_stream);
	this->read_defn_end(a_stream);
}

// Class Map member functions

Object_Tag_Format* Class_Map::get_format_for(int
	class_code)
{
	Object_Tag_Format* format;

	format = formats.item_at(class_code);
	return format;
}



