/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* STRING.CPP                                                */
/*************************************************************/

#include <iostream.h>
#include <strstream.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef CC_STRING_H
#include "string.h"
#endif

String::String(String& new_string)
{
	char* char_array = new_string;
	string_length = strlen(char_array);
	character_array = new char[string_length + 1];
	strcpy(character_array, char_array);
}

String::String(char* new_string)
{
	string_length = strlen(new_string);
	character_array = new char[string_length + 1];
	strcpy(character_array, new_string);
}

String::String(int integer_to_convert)
{
	char* temp_array = new char[15];

	sprintf(temp_array, "%i", integer_to_convert);
	string_length = strlen(temp_array);
	character_array = new char[string_length + 1];

	strcpy(character_array, temp_array);
	delete [] temp_array;
}

String::String(float float_to_convert)
{
	char* temp_array = new char[15];

	sprintf(temp_array, "%.2f", float_to_convert);
	string_length = strlen(temp_array);
	character_array = new char[string_length + 1];

	strcpy(character_array, temp_array);
	delete [] temp_array;
}


int String::as_integer()
{
	return atoi(character_array);
}

float String::as_float()
{
	return atof(character_array);
}

String& String::read_up_to(char delimiter)
{
	char* temp_array = new char[string_length + 1];
	istrstream input_string_stream(character_array, string_length);
	input_string_stream.get(temp_array, string_length + 1, delimiter);

	String* new_string;
	if (strlen(temp_array) == string_length)
		new_string = new String("");
	else
		new_string = new String(temp_array);
	delete [] temp_array;
	return *new_string;
}

String& String::break_at(char delimiter)
{
	String& substring_up_to = this->read_up_to(delimiter);
	int substring_size = strlen(substring_up_to);

	if (substring_size == 0) return substring_up_to;

	char* temp_array = new char[(string_length - substring_size) + 1];
	for (int i = substring_size ; i < string_length; i++)
		temp_array[i - substring_size] = character_array[i + 1];

	delete [] character_array;
	character_array = new char[strlen(temp_array) + 1];
	strcpy(character_array, temp_array);
	string_length = strlen(character_array);
	delete [] temp_array;

	return substring_up_to;
}

Boolean String::contains_string(char* substring)
{
	char* sub_array = strstr(character_array, substring);
	return (sub_array != NULL);
}

int String::locate_string(char* substring)
{
	char* sub_array = strstr(character_array, substring);
	if (sub_array == NULL)
		return -1;
	else
		return string_length - strlen(sub_array);
}

String& String::copy()
{
	String* new_string = new String(character_array);
	return *new_string;
}

String& String::copy_sequence(int sequence_start, int sequence_end)
{
	char* temp_array;
	int start_index, end_index;
	if (sequence_start < 0)
		start_index = 0;
	else
		start_index = sequence_start;
	if (sequence_end > string_length)
		end_index = string_length;
	else
		end_index = sequence_end;
	if (start_index == 0)
		{
		temp_array = new char[end_index + 1];
		strncpy(temp_array, character_array, end_index + 1);
		}
	else
		{
		temp_array = new char[end_index - start_index + 2];
		for (int i = start_index; i <= end_index; i++)
			temp_array[i - start_index] =
					character_array[i];
		temp_array[end_index - start_index + 2] = '\0';
		}
	String* new_string = new String(temp_array);
	return *new_string;
}



String& String::operator+ (String& a_string)
{
	int total_size = string_length + strlen(a_string) + 1;
	char* temp_array = new char[total_size];
	char* char_array = a_string;

	strcpy(temp_array, character_array);
	strcat(temp_array, char_array);

	String* new_string = new String(temp_array);
	delete [] temp_array;
	return *new_string;
}

String& String::operator+= (String& a_string)
{
	int total_size = string_length + strlen(a_string) + 1;
	char* temp_array = new char[total_size];
	char* char_array = a_string;

	strcpy(temp_array, character_array);
	strcat(temp_array, char_array);

	delete [] character_array;
	character_array = temp_array;
	string_length = strlen(character_array);
	return *this;
}

String& String::operator= (String& a_string)
{
	char* new_char_array = a_string;
	delete [] character_array;
	character_array = new char[strlen(new_char_array) + 1];
	strcpy(character_array, new_char_array);
	string_length = strlen(character_array);
	return *this;
}

String& String::operator= (const char* a_character_array)
{
	delete [] character_array;
	character_array = new char[strlen(a_character_array) + 1];
	strcpy(character_array, a_character_array);
	string_length = strlen(character_array);
	return *this;
}

int String::operator< (String& a_string)
{
	return(strcmp(character_array, (char*) a_string) < 0);
}

int String::operator> (String& a_string)
{
	return(strcmp(character_array, (char*) a_string) > 0);
}

int String::operator==(String& a_string)
{
	return(strcmp(character_array, (char*) a_string) == 0);
}

ostream& operator<< (ostream& os, String& a_string)
{
	os << a_string.character_array;
	return os;
}

