/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* TIME.H                                                    */
/*************************************************************/

#include <time.h>
#include <strstrea.h>

#ifndef CC_BOOLE_H
#include "cc_boole.h"
#endif

#ifndef CC_STRING_H
#include "string.h"
#endif

#ifndef CC_TIME_H
#define CC_TIME_H

class Time  {
public:

	// Constructors/destructors
	Time(){ time_of_day = time(NULL);}
	Time(time_t a_time) { time_of_day = a_time; }
	Time(int hours, int minutes = 0, int seconds = 0);
	~Time() { }

	// Implementors
	void set_time(int hours, int minutes, int seconds);
	int hours();
	int minutes();
	int seconds();

	String print_string();

	// Arithmetic
	Time add_time(int hour = 0, int minutes = 0, int seconds = 0);
	Time subtract_time(int hour = 0, int  minutes = 0,
				int seconds = 0);

	// Arithmetic operators
	friend Time operator+ (Time& time1, Time& time2);
	friend Time operator- (Time& time1, Time& time2);

	// Increment/decrement operators
	Time operator++(); 	        // prefix
	Time operator++(int ignored);   // postfix

	Time operator--(); 	        // prefix
	Time operator--(int ignored);	// postfix

	// Comparision operators
	friend Boolean operator< (Time& time1, Time& time2);
	friend Boolean operator> (Time& time1, Time& time2);
	friend Boolean operator== (Time& time1, Time& time2);

	// Conversion operator: converts to long
	long as_seconds();

	friend ostream& operator<< (ostream& os, Time& aTime);

private:
	time_t time_of_day;
	static long seconds_in_hour;
	static long seconds_in_minute;
};

#endif
