/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* TRN_VWS2.H                                                */
/*************************************************************/

#include <iostream.h>
#ifndef CON_VIWS_H
#include "con_viws.h"
#endif
#ifndef SALES_H
#include "sales.h"
#endif

#ifndef TRN_VWS2_H
#define TRN_VWS2_H

// Forward references
class Transaction_Selection_Holder;

// Name declarations
typedef String (Customer::*Customer_String_Function)();
typedef String (Transaction::*Transaction_Float_Function)();

// Console Display Boxes
typedef Console_Display_Box <Customer,
	Customer_String_Function>
		Customer_Console_Display_Box;

typedef Console_Display_Box <Transaction,
	Transaction_Float_Function>
		Transaction_Console_Display_Box;

// Class Declarations

class Transaction_Selection_Holder {
public:
	// Constructors/destructors
	Transaction_Selection_Holder() { }
	~Transaction_Selection_Holder() { }

	// Implementors
	void change_selection(int selection_index);

	// Accessors
	Transaction& get_selection()
		{ return *transaction_selection; }
	void set_selection(Transaction& a_transaction)
		 { transaction_selection = &a_transaction; }

	Customer& get_customer()
		{ return *a_customer; }
	void connect_to_customer(Customer& new_customer)
		{ a_customer = &new_customer; }

private:

	Transaction* transaction_selection;
	Customer* a_customer;
};


class Line_Item_Selection_Holder {
public:
	// Constructors/destructors
	Line_Item_Selection_Holder() { }
	~Line_Item_Selection_Holder() { }

	// Implementors
	void change_selection(int selection_index);

	// Accessors
	Line_Item& get_selection()
		{ return *line_item_selection; }
	void set_selection(Line_Item& new_selection)
		{ line_item_selection = &new_selection; }

	Transaction& get_transaction()
		{ return *a_transaction; }
	void connect_to_transaction(Transaction&
			new_transaction)
		{ a_transaction = &new_transaction;}

private:

	Line_Item* line_item_selection;
	Transaction* a_transaction;
};


class Transaction_Console_Menu : public Console_Menu {
public:
	// Constructors/destructors
	Transaction_Console_Menu() { a_transaction_holder =
		new Transaction_Selection_Holder(); }
	~Transaction_Console_Menu() { }

	// Implementors
	void make_items(Customer& new_customer);
	void execute_selection();

	// Accessors
	Customer& get_customer()
		{ return *a_customer; }
	void set_customer(Customer& new_customer)
		{ a_customer = &new_customer; }

	Transaction_Selection_Holder& get_trnasaction_holder()
		{ return *a_transaction_holder; }
	void connect_to_transaction_holder
		(Transaction_Selection_Holder& new_holder)
		{ a_transaction_holder = &new_holder; }
private:

	Customer* a_customer;
	Transaction_Selection_Holder* a_transaction_holder;
};


class Customer_Container {
public:

	// Constructors/destructors
	Customer_Container() { }
	~Customer_Container() {}

	// Implementors
	void make_name_box(Customer& a_customer);
	void make_address_box(Customer& a_customer);
	void make_transaction_console_menu(Customer& a_customer);
	void build_views(Customer& a_customer);

	void display_components();
	void display_customer_components();
	void connect_menu_to_holder
		(Transaction_Selection_Holder&
			a_transaction_holder);

private:

	// Data members
	Customer_Console_Display_Box* name_box;
	Customer_Console_Display_Box* address_box;
	Transaction_Console_Menu* transaction_menu;
};


class Line_Item_Console_Menu : public Console_Menu {
public:
	// Constructors/destructors
	Line_Item_Console_Menu() { }
	~Line_Item_Console_Menu() { }

	// Implementors
	void make_items(Transaction& new_transaction);
	void execute_selection();

	// Accessors
	Transaction& get_transaction()
		{ return *a_transaction; }
	void connect_to_transaction(Transaction& new_transaction)
		{ a_transaction = &new_transaction; }

	Line_Item_Selection_Holder& get_line_item_holder()
		{ return *a_line_item_holder; }
	void connect_to_line_item_holder
		(Line_Item_Selection_Holder& a_holder)
		{ a_line_item_holder = &a_holder; }

private:

	// Data members
	Transaction* a_transaction;
	Line_Item_Selection_Holder* a_line_item_holder;
};

class Transaction_Container {
public:

	// Constructors/destructors
	Transaction_Container() { }
	~Transaction_Container() {}

	// Implementors
	void make_total_box();
	void make_line_item_console_menu();
	void build_views();

	void display_components();
	void change_displayed_transaction
		(Transaction& selected_transaction);
	void connect_menu_to_holder
		(Line_Item_Selection_Holder&
			a_line_item_selection_holder);

private:

	// Data members
	Transaction_Console_Display_Box* total_box;
	Line_Item_Console_Menu* line_item_menu;
};








#endif
