/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* TRN_VWS5.CPP                                              */
/*************************************************************/

#ifndef TRN_VWS5_H
#include "trn_vws5.h"
#endif

//Transaction_Selection_Holder member functions

void Transaction_Selection_Holder::change_selection(int selection_index)
{
	// (List indices start at zero).
	int list_selection = selection_index - 1;
	Transaction_List& transactions =
		a_customer->get_transactions();
	transaction_selection =
		transactions.item_at(list_selection);
	a_transaction_container->
		change_displayed_transaction(*transaction_selection);
}

void Line_Item_Selection_Holder::change_selection(int selection_index)
{
	int list_selection = selection_index - 1;
	Line_Item_List& line_items =
			a_transaction->get_line_items();
	line_item_selection =
		line_items.item_at(list_selection);
	a_line_item_container->
		change_displayed_line_item(*line_item_selection);
}

// Member functions for Transaction_Console_Menu

void Transaction_Console_Menu::make_items
	(Customer& a_customer)
{
	menu_size = 0;
	Transaction_List& transactions =
		a_customer.get_transactions();
	while(!transactions.end_of_list())
		{
		String transaction_description =
			transactions.current()->get_date_time();
		this->add_item(transaction_description);
		transactions++;
		}
	this->set_customer(a_customer);
}

void Transaction_Console_Menu::execute_selection()
{
	if (selection == 0)
		return;
	a_transaction_holder->change_selection(selection);
}

// Customer_Container member functions

void Customer_Container::make_name_box(Customer&
	a_customer)
{
	name_box = new Customer_Console_Display_Box();

	name_box->set_value_function(&Customer::get_name);
	name_box->set_label_string("Name:");
	name_box->set_object(a_customer);
}

void Customer_Container::make_address_box(Customer& a_customer)
{
	address_box = new Customer_Console_Display_Box();

	address_box->set_value_function(&Customer::get_address);
	address_box->set_label_string("Address: ");
	address_box->set_object(a_customer);
}

void Customer_Container
	::make_transaction_console_menu(Customer& a_customer)
{
	transaction_menu = new Transaction_Console_Menu();

	transaction_menu->make_items(a_customer);
	transaction_menu->add_terminator("Quit");
	transaction_menu->add_prompt("Select a transaction:  ");
}


void Customer_Container::build_views
	(Customer& a_customer)
{
	this->make_name_box(a_customer);
	this->make_address_box(a_customer);
	this->make_transaction_console_menu(a_customer);
}

void Customer_Container::connect_menu_to_holder
	(Transaction_Selection_Holder&
		a_transaction_holder)
{
	transaction_menu->
		connect_to_transaction_holder(a_transaction_holder);
}

void Customer_Container::display_customer_components()
{
	char* border = "-------------------------";

	cout << border << endl;
	name_box->display_value();
	cout << endl;
	address_box->display_value();
	cout << endl << border << endl;
}

void Customer_Container::display_components()
{
	this->display_customer_components();
	while(transaction_menu->get_user_selection() != 0)
		{
		transaction_menu->execute_selection();
		this->display_customer_components();
		}
}

// Line_Item_Console_Menu member function

void Line_Item_Console_Menu::make_items
	(Transaction& new_transaction)
{
	Line_Item_List& line_items =
		new_transaction.get_line_items();
	menu_size = 0;
	String item_number_string;

	while(!line_items.end_of_list())
		{
		item_number_string = String("Line Item: ") +
			String(line_items.current()
				->get_line_number());
		this->add_item(item_number_string);
		line_items++;
	}
	this->connect_to_transaction(new_transaction);
}


void Line_Item_Console_Menu::execute_selection()
{
	if (selection == 0)
		return;
	a_line_item_holder->change_selection(selection);
}

// Transaction_Container member functions

void Transaction_Container::make_total_box()
{
	total_box =
		new Transaction_Console_Display_Box();
	total_box->set_value_function(&Transaction::total_string);
	total_box->set_label_string("Transaction Total:");
}

void Transaction_Container::make_line_item_console_menu()
{
	line_item_menu = new Line_Item_Console_Menu();
	line_item_menu->add_prompt("Select a line_item:  ");
}

void Transaction_Container::build_views()
{
	this->make_total_box();
	this->make_line_item_console_menu();
}

void Transaction_Container::change_displayed_transaction
	(Transaction& selected_transaction)
{
	line_item_menu->make_items(selected_transaction);
	line_item_menu->add_terminator("Quit");
	total_box->set_object(selected_transaction);

	a_line_item_selection_holder
		->connect_to_transaction(selected_transaction);

	this->display_components();

}

void Transaction_Container::connect_menu_to_holder
	(Line_Item_Selection_Holder& a_line_item_holder)
{
	line_item_menu->
		connect_to_line_item_holder(a_line_item_holder);
}

void Transaction_Container::display_components()
{
	cout << endl;
	total_box->display_value();
	cout << endl;
	if (line_item_menu->get_user_selection() != 0)
		line_item_menu->execute_selection();
}

void Transaction_Container::connect_to_line_item_selection_holder
	(Line_Item_Selection_Holder& a_line_item_holder)
{
	this->connect_menu_to_holder(a_line_item_holder);
	a_line_item_selection_holder = &a_line_item_holder;
}

// Line Item Container member functions

void Line_Item_Container::make_total_box()
{
	total_box = new Line_Item_Console_Display_Box();

	total_box->set_value_function(&Line_Item::total_string);
	total_box->set_label_string("Line_Item Total:");
}

void Line_Item_Container::make_quantity_box()
{
	quantity_box = new Line_Item_Console_Display_Box();

	quantity_box->
		set_value_function(&Line_Item::quantity_string);
	quantity_box->set_label_string("Line_Item Quantity:");
}

void Line_Item_Container::build_views()
{
	this->make_total_box();
	this->make_quantity_box();
}

void Line_Item_Container::change_displayed_line_item
	(Line_Item& selected_line_item)
{
	total_box->set_object(selected_line_item);
	quantity_box->set_object(selected_line_item);
	this->display_components();
}


void Line_Item_Container::display_components()
{
	cout << endl;
	total_box->display_value();
	cout << "   ";
	quantity_box->display_value();
	cout << endl;
}




