/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* ARRAYCOL.H                                                */
/*************************************************************/

#include<stdlib.h>

#ifndef ARRAY__COL_H
#define ARRAY_COL_H

// Forward reference
template<class Item_Type>
class Array_Collection_Iterator;


template<class Item_Type>
class Array_Collection {
public:
	Array_Collection ();
	Array_Collection(int size);
	~Array_Collection() { delete [] item_array; }

	Item_Type& operator[] (int subscript);
	int size() { return array_size; }
	int tally() { return number_of_items; }

	void add(const Item_Type& new_item);
	void add_at(const Item_Type& new_item, int location);

	void reset() { position = 0; }

	int operator++ ();	// Prefix increment
//	int operator++ (int);	// Postfix increment

private:
	int number_of_items;
	int array_size;
	int position;
	Item_Type* item_array;
};

template<class Item_Type>
Array_Collection<Item_Type>::Array_Collection()
{
	Array_Collection<Item_Type>::Array_Collection(10);
}


template<class Item_Type>
Array_Collection<Item_Type>::Array_Collection(int size)
{
	position = 0;
	number_of_items = 0;
	array_size = size;
	item_array = new Item_Type[array_size];
}

template<class Item_Type>
void Array_Collection<Item_Type>::add(const Item_Type& new_item)
{
	if (number_of_items == array_size)
		{
		cerr << "Can't add to full array\n";
		exit(-1);
		}
	item_array[number_of_items] = new_item;
	number_of_items++;
}

template<class Item_Type>
void Array_Collection<Item_Type>::add_at(const Item_Type& new_item,
						int location)
{
	if ((location >= array_size) || (location < 0))
		{
		cerr << "Subscript out of bounds for array\n";
		exit(-1);
		}
	item_array[location] = new_item;
	number_of_items = (number_of_items <= location ?
				location + 1 : number_of_items);
}

template<class Item_Type>
Item_Type& Array_Collection<Item_Type>::operator[] (int subscript)
{
	if ( (subscript >= array_size) || (subscript < 0))
		{
		cerr << "Subscript out of bounds for array\n";
		exit(-1);
		}
	return item_array[subscript];
}


template<class Item_Type>
int Array_Collection<Item_Type>::operator++ ()
{
	if (position < number_of_items)
		{
		int location = position;
		position++;
		return location;
		}
	else return -1;
}

#endif


/*
template<class Item_Type>
class Array_Collection_Iterator {
public:

	// Constructors/destructors
	Array_Collection_Iterator(Array_Collection<Item_Type>& new_array_)
				{ array_collection = &new_array;
					position = 0; }
	~Array_Collection_Iterator() { }

	// Overloaded function call operator
	int operator() ()
		{ if (position < array_collection->number_of_items;)
			{
			int location = position;
			position++;
			return location;
			}
		else return -1;
		}
	void reset() { position = 0; }

private:

	// Data members
	Array_Collection<Item_Type>* array_collection;
	int position;
};
*/