/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* FLOW2.H                                                   */
/*************************************************************/

#include <iostream.h>
#include <stdlib.h>
#include <time.h>
#include <dos.h>

#include "string.h"
#include "olist.h"
#include "cc_boole.h"

#ifndef FLOW2_H
#define FLOW2_H

// Forward reference
class Lane;

typedef Ordered_List<Lane*> Lane_List;

class Lane {
public:

	// Constructors/destructors
	Lane(float max_rate);
	Lane(String a_name = "", float max_rate = 0.0);
	~Lane() { }

	// Implementors
	//Rate adjustment
	Boolean adjust_flow_rate(float desired_flow_rate,
		int transition_duration,
		int duration,
		Boolean only_when_requested = false);
	Boolean adjust_flow_rate_to_max(int transition_duration,
	       int duration,
	       Boolean only_when_requested = false);

	void cut_flow_rate_in_half();

	// Safety precautions
	void close_crossing_lanes(int transition_duration);

	// Lane requests
	void clear_requests() { requested = false;}
	void request() { requested = true; }
	void get_any_waiting_requests();

	// Demo functions
	void close();
	void open ();
	void wait(int duration_in_seconds)
		{ delay(duration_in_seconds * 1000); }

	void print_flow_change(float desired_rate, int duration);

	// Accessors
	float get_current_flow_rate()
		{ return current_flow_rate; }
	void set_current_flow_rate(float new_value)
		{ current_flow_rate = new_value; }

	float get_max_flow_rate() { return max_flow_rate; }
	void set_max_flow_rate(float new_value)
		{ max_flow_rate = new_value; }

	Lane_List& get_crossing_lanes()
		{ return crossing_lanes; }

	void connect_to_crossing_lane(Lane& new_lane);
	Boolean disconnect_with_crossing_lane(Lane& aLane);

	Boolean get_requested() { return requested; }
	void set_requested(Boolean new_value)
		{ requested = new_value; }
	String get_name() { return name; }
	void set_name(String new_name) { name = new_name; }

	// Overloaded operators
	Boolean operator== (Lane& traffic_lane);
	friend ostream& operator<< (ostream& os, Lane& traffic_lane);

private:
	// Simulating
	int next_random_number() { return random(100); }

	// Data members
	float current_flow_rate;
	float max_flow_rate;
	Lane_List crossing_lanes;
	Boolean requested;

	String name;
};

#endif