/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* FLOW2.CPP                                                 */
/*************************************************************/

#include <iostream.h>
#ifndef FLOW3_H
#include "flow3.h"
#endif

// Lane constructors

Lane::Lane(float max_rate)
{
	current_flow_rate = 0.0;
	max_flow_rate = max_rate;
	requested = false;
}

Lane::Lane(String a_name, float max_rate)
 {
	current_flow_rate = 0.0;
	max_flow_rate = max_rate;
	name = a_name;
	requested = false;
}

// Lane connection accessors

void Lane::connect_to_crossing_lane(Lane& traffic_lane)
{
	if (!crossing_lanes.includes(&traffic_lane))
		crossing_lanes.add(&traffic_lane);
}

Boolean Lane::disconnect_with_crossing_lane(Lane& traffic_lane)
{
	return(crossing_lanes.remove(&traffic_lane));

}

// Lane member functions and operators

ostream& operator<< (ostream& os, Lane& a_lane)
{
	os << endl
	   << "Current_flow_rate:  " << a_lane.current_flow_rate
	   << endl
	   << "Max_flow_rate:  " << a_lane.max_flow_rate
	   << endl
	   << "Requested:  " << a_lane.requested
	   << endl;
	return os;
}

void Lane::close_crossing_lanes(int transition_duration)
{
	Lane* next_lane;
	cout << name << ".  " << "I'm closing my crossing lanes in "
		<< transition_duration << " seconds." << endl;
	while(!crossing_lanes.end_of_list())
		{
		next_lane = crossing_lanes.current();
		next_lane->cut_flow_rate_in_half();
		crossing_lanes++;
		}
	this->wait(transition_duration / 2);
	crossing_lanes.reset();
	while(!crossing_lanes.end_of_list())
		{
		next_lane = crossing_lanes.current();
		next_lane->close();
		crossing_lanes++;
		}
	this->wait(transition_duration / 2);
}

void Lane::cut_flow_rate_in_half()
{
	current_flow_rate /= 2.0;
}

void Lane::open ()
{
	cout << name << ".  " << "I'm open." << endl;

	current_flow_rate = max_flow_rate;
}

void Lane::close()
{
	cout << name << ".  " <<
		"I'm closed." << endl;

	current_flow_rate = 0.0;
}

template<class Type>
inline Type min(Type magnitude1, Type magnitude2)
{
	return( magnitude1 < magnitude2 ? magnitude1 :
		magnitude2);
}

Boolean Lane::adjust_flow_rate(float desired_rate,
	 int transition_duration,
	 int duration,
	 Boolean only_when_requested)
{
	if (only_when_requested && (!this->get_requested()))
		return false;

	this->close_crossing_lanes(transition_duration);
	current_flow_rate = min(desired_rate, max_flow_rate);
	this->print_flow_change(current_flow_rate, duration);
	this->wait(duration);
	return true;
}


Boolean Lane::adjust_flow_rate_to_max
		(int transition_duration,
		 int duration,
		 Boolean only_when_requested)
{
	return(this->adjust_flow_rate(max_flow_rate,
		transition_duration,
		duration,
		only_when_requested));
}

void Lane::print_flow_change(float desired_rate, int
	duration)
{
	   cout << name << ".  " << "I'm adjusting to "
		<< desired_rate
		<< " for " << duration
		<< " seconds."
		<< endl;
}

void Lane::get_any_waiting_requests()
{
	if (this->next_random_number() > 50)
		this->request();
}

// Step initialization

float Step::flow_rate_maximum = 65.0;

// Step constructors

Step::Step()
{
	transition_duration = 0;
	duration = 0;
	desired_flow_rate = 0.0;
	only_when_requested = false;
}

Step::Step(float flow_rate, int a_transition_duration,
		int step_duration, Boolean request_flag)
{
	transition_duration = a_transition_duration;
	duration = step_duration;
	desired_flow_rate = flow_rate;
	only_when_requested = request_flag;
}

// Step member functions and operators

ostream& operator<< (ostream& os, Step& a_step)
{
	os << "Transition_duration:  "
	   << a_step.transition_duration
	   << endl
	   << "Duration:  " << a_step.duration
	   << endl
	   << "Desired_flow_rate:  "
	   << a_step.desired_flow_rate
	   << endl
	   << "Requested:  " << a_step.only_when_requested
	   << endl
	   << "Lane:  " << *a_step.a_lane
	   << endl;
	return os;
}

inline void Step::set_desired_flow_rate_to_max()
{
	desired_flow_rate = flow_rate_maximum;
}

Boolean Step::desired_flow_rate_is_max()
{
	if (desired_flow_rate == flow_rate_maximum)
		return true;
	else
		return false;
}

void Step::run() {
	if (this->desired_flow_rate_is_max())
		a_lane->adjust_flow_rate_to_max(transition_duration,
			       duration,
			       only_when_requested);
	else
		a_lane->adjust_flow_rate(desired_flow_rate,
			       transition_duration,
			       duration,
			       only_when_requested);
}

