/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* TIME.CPP                                                  */
/*************************************************************/

#include <iostream.h>
#include <ctype.h>
#ifndef CC_TIME_H
#include "time.h"
#endif

long Time::seconds_in_hour = 3600;
long Time::seconds_in_minute = 60;

Time::Time(int hour, int minutes, int seconds)
{
	this->set_time(hour, minutes, seconds);
}

void Time::set_time(int hour, int minutes, int seconds)
{
	time_t now = time(NULL);
	struct tm* time_now = localtime(&now);
	struct tm time_struct = {0};
	time_t temp_time;
	time_struct.tm_mday = time_now->tm_mday;
	time_struct.tm_mon = time_now->tm_mon;
	time_struct.tm_year = time_now->tm_year;
	time_struct.tm_hour = hour;
	time_struct.tm_min = minutes;
	time_struct.tm_sec = seconds;
	temp_time = mktime(&time_struct);
	if (temp_time != (time_t)-1)
		time_of_day = temp_time;
	else
		cout << "bad time, dummy" ;
}

int Time::hours ()
{
	struct tm* time_struct = localtime(&time_of_day);
	return time_struct->tm_hour;
}

int Time::minutes ()
{
	struct tm* time_struct = localtime(&time_of_day);
	return time_struct->tm_min;
}

int Time::seconds ()
{
	struct tm* time_struct = localtime(&time_of_day);
	return time_struct->tm_sec;
}

String Time::print_string()
{
	char* time_string = new char[30];
	struct tm* time_struct = localtime(&time_of_day);

	strftime(time_string, 30, "%X", time_struct);
	return String(time_string);
}

// Arithmetic functions

Time Time::add_time(int hour, int minutes, int seconds)
{
	Time time_to_add(hour, minutes, seconds);
	return *this + time_to_add;
}

Time Time::subtract_time(int hour, int minutes, int seconds)
{
	Time time_to_subtract(hour, minutes, seconds);
	return *this - time_to_subtract;
}

// Arithmetic operators

Time operator+(Time& time1, Time& time2)
{
	long seconds_in_time = time1.time_of_day;
	seconds_in_time += time2.as_seconds();
	return Time((time_t)seconds_in_time);
}

Time operator-(Time& time1, Time& time2)
{
	long seconds_in_time = time1.time_of_day;
	seconds_in_time -= time2.as_seconds();
	return Time((time_t)seconds_in_time);
}

// Increment/decrement operators

Time Time::operator++()
{
	++time_of_day;
	return *this;
}

Time Time::operator++(int)
{
	time_of_day++;
	return *this;
}

Time Time::operator--()
{
	--time_of_day;
	return *this;
}

Time Time::operator--(int)
{
	time_of_day--;
	return *this;
}

// Comparison operators

Boolean operator< (Time& time1, Time& time2)
{
	return Boolean(time1.time_of_day < time2.time_of_day);
}

Boolean operator> (Time& time1, Time& time2)
{
	return Boolean(time1.time_of_day > time2.time_of_day);
}

Boolean operator== (Time& time1, Time& time2)
{
	return Boolean(time1.time_of_day == time2.time_of_day);
}

// Conversion operator

long Time::as_seconds()
{
	struct tm* time_struct = localtime(&time_of_day);
	long seconds_in_time = 0;
	int hour = (time_struct->tm_hour >= 12) ?
			time_struct->tm_hour - 12 :
			time_struct->tm_hour;
	seconds_in_time += (hour * seconds_in_hour);
	seconds_in_time += (time_struct->tm_min * seconds_in_minute);
	seconds_in_time += time_struct->tm_sec;
	return seconds_in_time;
}

// Insertion operator

ostream& operator<< (ostream& os, Time& aTime)
{
	os << aTime.print_string();
	return os;
}

