/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* TRAFVWS2.CPP                                              */
/*************************************************************/

#ifndef FLOW7_H
#include "flow7.h"
#endif
#ifndef TRAFVWS2_H
#include "trafvws2.h"
#endif

//Member functions for Display_Color

const space = ' ';

ostream& operator<< (ostream& os, Display_Color& display_color)
{
	os << display_color.get_name();
	return os;
}

Boolean Display_Color::operator== (Display_Color& display_color)
{
	return (name == display_color.get_name());
}

//Member functions for Display_Shape

ostream& operator<< (ostream& os, Display_Shape& display_shape)
{
	os << display_shape.get_name();
	return os;
}

Boolean Display_Shape::operator== (Display_Shape& display_shape)
{
	return (name == display_shape.get_name());
}

// Member functions for Light_Box

Light_Box::Light_Box()
{
	color = Display_Color("Red");
	shape = Display_Shape("Circle");
	location = 0;
}

Light_Box::Light_Box(Display_Color color_value,
			Display_Shape geometric_shape,
			Rule light_display_rule)
{
	color = color_value;
	shape = geometric_shape;
	display_rule = light_display_rule;
	location = 0;
}

// Initialization of static class members
// Display_Color Light_Box::background_color;

Boolean Light_Box::check_if_on()
{
	// Invoke the pointer to member function stored in
	// the display_rule data member
	Boolean result =
		(lane_view_container->*display_rule)(*lane);
	return result;
}
                                                
void Light_Box::display_value()
{
	if (!this->check_if_on())
		return;
	else
		{
		cout.width(location);
		String display = color.get_name() + String(" ")
			+ shape.get_name();
		cout << display;
		cout.width(0);
		}
}


ostream& operator<< (ostream& os, Light_Box& light_box)
{
	os << light_box.get_color() << "\t"
	   << light_box.get_shape() << "\t";
	return os;
}


void Light_Box::update_display()
{
	lane_view_container->display_components();
}

// Lane View Container

//Initialization of static members
Display_Color Lane_View_Container::stop_color
	("Red");
Display_Color Lane_View_Container::caution_color
	("Yellow");
Display_Color Lane_View_Container::go_color
	("Green");
Display_Shape Lane_View_Container::light_box_shape
	("Circle");


// Member functions for Lane_View_Container

Boolean Lane_View_Container::stop_light_rule(Lane& traffic_lane)
{
	return (traffic_lane.get_current_flow_rate() == 0);
}

Boolean Lane_View_Container::caution_light_rule(Lane& traffic_lane)
{

//	return ((0 < traffic_lane.get_current_flow_rate()) &&
//		(traffic_lane.get_current_flow_rate()
//			 < traffic_lane.get_max_flow_rate() ));
	if ((0.0 < traffic_lane.get_current_flow_rate())
		&&
		(traffic_lane.get_current_flow_rate() <
			traffic_lane.get_max_flow_rate() ))
		return true;
	else
		return false;
}

Boolean Lane_View_Container::go_light_rule(Lane& traffic_lane)
{
	return (traffic_lane.get_current_flow_rate() ==
			traffic_lane.get_max_flow_rate());
}

int Lane_View_Container::stop_location()
{
	int location;
	location = stop_color.get_name().size()
		+ 1
		+ light_box_shape.get_name().size()
		+ 5;
	return location;
}

int Lane_View_Container::caution_location()
{
	int location;
	location = caution_color.get_name().size()
		+ 1
		+ light_box_shape.get_name().size()
		+ 5
		+ this->stop_location();
	return location;
}

int Lane_View_Container::go_location()
{
	int location;

	location = go_color.get_name().size()
		+ 1
		+ light_box_shape.get_name().size()
		+ 5
		+ this->caution_location();
	return location;
}

Light_Box& Lane_View_Container::make_light_box
	(Lane& traffic_lane, Display_Color color_value,
	 int location, Rule a_rule)
{
	Light_Box* light_box = new Light_Box(color_value,
			light_box_shape,
			a_rule);

	light_box->connect_to_lane(traffic_lane);
	light_box->set_location(location);
	light_box->connect_to_lane_view_container(*this);

	return *light_box;
}


Light_Box& Lane_View_Container::build_go_light(Lane&
	traffic_lane)
{
	return (this->make_light_box(traffic_lane,
		     this->get_go_color(),
		     this->go_location(),
		     &Lane_View_Container::go_light_rule));
}

Light_Box& Lane_View_Container::build_stop_light(Lane& traffic_lane)
{
	return (this->make_light_box(traffic_lane,
				     stop_color,
				     this->stop_location(),
				     &Lane_View_Container::stop_light_rule));
}

Light_Box& Lane_View_Container::build_caution_light(Lane& traffic_lane)
{
	return (this->make_light_box(traffic_lane,
				     caution_color,
				     this->caution_location(),
				     &Lane_View_Container::caution_light_rule));
}

void Lane_View_Container::add_view(Light_Box& light_box)
{
	components.add(&light_box);
}

void Lane_View_Container::open_light(Lane& traffic_lane,
	String lane_name)
{
	// Builf the light boxes.
	Light_Box& stop_light =
		this->build_stop_light(traffic_lane);
	Light_Box& caution_light =
		this->build_caution_light(traffic_lane);
	Light_Box& go_light =
		this->build_go_light(traffic_lane);

	// Arrange them inside me.
	this->add_view(stop_light);
	this->add_view(caution_light);
	this->add_view(go_light);

	this->set_label(lane_name);

	// Arrange the light boxes for the lane.
	traffic_lane.connect_to_traffic_light(stop_light);
	traffic_lane.connect_to_traffic_light(caution_light);
	traffic_lane.connect_to_traffic_light(go_light);
}

void Lane_View_Container::display_components()
{
	Light_Box* light_box_component;
	cout << label;
	while(!components.end_of_list())
		{
		light_box_component = components.current();
		light_box_component->display_value();
		components++;
		}
	components.reset();
	cout << endl;
}

