/*************************************************************/
/* OBJECT-ORIENTED PROGRAMMING                               */
/* Peter Coad / Jill Nicola                                  */
/* Source Code Diskette                                      */
/* (c)1993  Object International, Inc.  All rights reserved. */
/*************************************************************/

/*************************************************************/
/* TRAFVWS2.H                                                */
/*************************************************************/

#ifndef TRAFVWS2_H
#define TRAFVWS2_H

#include "string.h"

// Forward references
class Lane;

class Display_Color {
public:
	Display_Color(){ be_background(); }
	Display_Color(String color_name){ name = color_name; }
	~Display_Color() { }

	//Implementors
	void be_red(){name = "Red";}
	void be_blue(){name = "Blue";}
	void be_green(){name = "Green";}
	void be_yellow(){name = "Yellow";}
	void be_white(){name = "White";}
	void be_black(){name = "Black";}
	void be_background(){name = " ";}

	//Accessors
	String get_name(){ return name; }
	void set_name(String color_name){ name = color_name;}

	// Overloaded operators
	friend ostream& operator<< (ostream& os,
			Display_Color& display_color);
	Boolean operator== (Display_Color& display_color);

private:

	String name;
};

class Display_Shape {
public:
	Display_Shape(){ be_square(); };
	Display_Shape(String shape_name){name = shape_name;}
	~Display_Shape() { }

	//Implementors
	void be_circle(){name = "Circle";}
	void be_square(){name = "Square";}
	void be_triangle(){name = "Triangle";}
	void be_ellipse(){name = "Ellipse";}

	//Accessors
	String get_name(){return name;}

	// Overloaded operators
	friend ostream& operator<< (ostream& os,
		Display_Shape& display_shape);
	Boolean operator== (Display_Shape& display_color);

private:

	String name;
};

// Forward declaration
class Lane_View_Container;
// Make Rule a synonym type for "pointer to member function of
// Lane_View_Container taking a Lane* argument and returning a Boolean."
typedef Boolean (Lane_View_Container::*Rule)(Lane& traffic_lane);

class Light_Box {
public:

	// Constructors/destructors
	Light_Box();
	Light_Box(Display_Color color_value,
		Display_Shape geometric_shape,
		Rule light_display_rule);
	~Light_Box() { }

	// Implementors

	// Displaying
	void display_value();
	void update_display();

	//Testing
	Boolean check_if_on();

	// Accessors
	Display_Color get_color() { return color; }
	void set_color(Display_Color& new_value)
		{ color = new_value; }

	Display_Shape get_shape() { return shape; }
	void set_shape(Display_Shape& new_value)
		{ shape = new_value; }
	void set_location(int new_location) {location =
		new_location;}
	int get_location() { return location; }

	Rule get_display_rule() { return display_rule; }
	void set_display_rule(Rule new_value)
		{ display_rule = new_value; }

	Lane& get_lane() { return *lane; }
	void connect_to_lane(Lane& new_value) { lane =
		&new_value; }

	Lane_View_Container& get_lane_view_container()
		{ return *lane_view_container; }
	void connect_to_lane_view_container(Lane_View_Container&
		new_value)
		{ lane_view_container = &new_value; }

	// Overloaded operators for Light_Box
	friend ostream& operator<< (ostream& os, Light_Box& light_box);

private:

	// Data members
	Display_Color color;
	Display_Shape shape;
	int location;
	Rule display_rule;
	Lane* lane;
	Lane_View_Container* lane_view_container;
};

class Lane_View_Container {
public:

	// Constructors/destructors
	Lane_View_Container () { }
	~Lane_View_Container () { }

// Implementors
	//Build views
	Light_Box& build_stop_light(Lane& traffic_lane);
	Light_Box& build_caution_light(Lane& traffic_lane);
	Light_Box& build_go_light(Lane& traffic_lane);

	//Make Views
	Light_Box& make_light_box(Lane& traffic_lane,
			Display_Color color_value,
			int location,
			Rule aRule);

	//Opening
	void open_light(Lane& traffic_lane, String lane_label);

	//Colors
	Display_Color get_stop_color() { return stop_color; }
	Display_Color get_caution_color() { return caution_color; }
	Display_Color get_go_color() { return go_color; }

	//Shapes
	Display_Shape get_light_box_shape() {return light_box_shape;}

	//Rules
	Boolean stop_light_rule(Lane& traffic_lane);
	Boolean caution_light_rule(Lane& traffic_lane);
	Boolean go_light_rule(Lane& traffic_lane);

	//Layout
	void display_components();
	void add_view(Light_Box& light_box);
	int stop_location();
	int caution_location();
	int go_location();

	//Accessors
	String get_label() {return label;}
	void set_label(String new_label) {label = new_label;}


private:

	//Data members
	Light_Box_List components;
	String label;
	static Display_Color stop_color;
	static Display_Color caution_color;
	static Display_Color go_color;
	static Display_Shape light_box_shape;
};

#endif




