'OBJECT-ORIENTED PROGRAMMING
Peter Coad / Jill Nicola
Source Code Diskette
(C) 1993  Object International, Inc.  All rights reserved.
'!

'OBJTBL.ST'!

Object subclass: #ObjectTable
	instanceVariableNames: 'objects classMap '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Transaction Formats'!


!ObjectTable methodsFor: 'initialize-release'!

initialize
	objects := Dictionary new! !

!ObjectTable methodsFor: 'accessing'!

classMap	
	 "Return the current value of classMap "

	^classMap!

classMap:  newValue
	 "Set the  value of classMap "

	classMap :=  newValue!

objects
	 "Return the current value of objects "

	^objects!

objects:  newValue
	 "Set the  value of objects "

	objects :=  newValue! !

!ObjectTable methodsFor: 'connections'!

connectToObject: object
	objects add: object!

disconnectWithObject: object 
	objects remove: object ifAbsent: []! !

!ObjectTable methodsFor: 'object creating'!

createObject: className 
	^classMap makeObjectInClass: className! !

!ObjectTable methodsFor: 'object finding'!

containsObject: objectID 
	^(self objectAt: objectID) notNil!

objectAt: objectID 
	"Return the object associated with the object id, or 
	return nil, if I don't contain the object."

	^objects at: objectID ifAbsent: []! !

!ObjectTable methodsFor: 'object registering'!

registerObject: object at: objectID
	^objects at: objectID put: object! !

!ObjectTable methodsFor: 'storing'!

saveToFile: fileNameString
	|  fileStream  |
	Cursor write
		showWhile:
			[fileStream :=
				(Filename named: fileNameString)
					writeStream.
			self storeOn: fileStream.
			self storeObjectsOn: fileStream.
			fileStream close]!

storeObjectsOn: stream
	objects values do:
		[:object | object storeOn: stream using: self]!

storeOn: stream
	| format |
	format := self classMap getFormatFor: self class name.
	format store: self on: stream! !

!ObjectTable methodsFor: 'restoring'!

readFromFile: fileNameString
	|  fileStream  |
	Cursor read
		showWhile:
			[fileStream :=
				(Filename named: fileNameString)
					readStream.
			self restoreFrom: fileStream.
			self restoreObjectsFrom: fileStream.
			fileStream close]!

restoreFrom: fileStream
	| format |
	format := self classMap getFormatFor: self class name.
	format restore: self from: fileStream!

restoreObjectsFrom: fileStream
	objects values do:
		[:each | each restoreFrom: fileStream using: self]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ObjectTable class
	instanceVariableNames: ''!


!ObjectTable class methodsFor: 'instance creation'!

new
	^super new initialize! !