'OBJECT-ORIENTED PROGRAMMING
Peter Coad / Jill Nicola
Source Code Diskette
(C) 1993  Object International, Inc.  All rights reserved.
'!

'OBTAGFMT.ST'!

Object subclass: #ObjectTagFormat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Transaction Formats'!


!ObjectTagFormat methodsFor: 'storing'!

store: anObject on: stream 
	"my format policy"

	self storeDefnStartOn: stream.
	self storeClassNameFor: anObject on: stream.
	self storeAttributesFor: anObject on: stream.
	self storeConnectionsFor: anObject on: stream.
	self storeDefnEndOn: stream!

storeAttributesFor: anObject on: stream 
	"Subclasses must implement. 
	Raise an error signal if this method is invoked."

	self subclassResponsibility!

storeClassNameFor: anObject on: stream 
	self
		storeTag: 'className'
		value: anObject class name
		on: stream!

storeConnectionsFor: anObject on: stream 
	"Optional. Subclasses need to implement only if they
	have connected objects that need saving."

	^self!

storeDefnEndOn: stream
	self storeTag: '\ObjectDefn' on: stream.
	stream cr!

storeDefnStartOn: stream
	self storeTag: 'ObjectDefn' on: stream.
	stream cr! !

!ObjectTagFormat methodsFor: 'restoring'!

readDefnEndFrom: stream
	self restoreTag: '\ObjectDefn' from: stream.
	stream atEnd ifFalse: [stream next].!

readDefnStartFrom: stream
	self restoreTag: 'ObjectDefn' from: stream.
	stream atEnd ifFalse: [stream next]!

restore: anObject from: stream using: objectTable 
	self readDefnStartFrom: stream.
	self restoreClassNameFrom: stream.
	self restoreAttributesFor: anObject from: stream.
	self restoreConnectionsFor: anObject 
		from: stream using: objectTable.
	self readDefnEndFrom: stream!

restoreAttributesFor: anObject on: stream 
	^self subclassResponsibility!

restoreClassNameFrom: stream 
	^self restoreTagValue: 'className' from: stream!

restoreConnectionsFor: anObject from: stream using: objectTable
	"Optional"! !

!ObjectTagFormat methodsFor: 'storing-private'!

storeTag: tagName on: stream 
	stream nextPutAll: '<' , tagName , '>'!

storeTag: tagName value: tagValue on: stream 
	self storeTag: tagName on: stream.
	self storeTagValue: tagValue on: stream!

storeTagValue: tagValue on: stream 
	"Write a value on stream. Any < characters in 
	tagValue are stored as <<."

	| readStream |
	readStream := tagValue readStream.
	[readStream atEnd]
		whileFalse: 
			[readStream peek = $<
				ifTrue: [stream nextPut: $<].
			stream nextPut: readStream next].
	stream cr! !

!ObjectTagFormat methodsFor: 'restoring-private'!

restoreTag: tagName from: stream
	"Read a tag from stream. If the tag read does not
	match the expected tag name, reset the stream to
	original position, and return nil. Otherwise
	return tag name."

	| writeStream  position |
	position := stream position.
	stream next.
	writeStream := WriteStream on: (String new: 300).
	writeStream nextPutAll: (stream upTo: $>).
	(writeStream contents sameAs: tagName)
		ifFalse:
			[stream position: position.
			^nil].
	^tagName!

restoreTagValue: tagName from: stream
	(self restoreTag: tagName from: stream) isNil ifTrue:
		[^nil].
	^self restoreTagValueFrom: stream!

restoreTagValueFrom: stream
	"Read a value from stream. Any < characters in
	tagValue are stored as <<, so drop extra one, and keep
	reading. Otherwise quit when I hit < by itself, and skip
	back one to put < back on stream."

	| writeStream done |
	writeStream := WriteStream on: (String new: 300).
	done := false.
	[done or: [stream atEnd]]
		whileFalse:
			[writeStream nextPutAll: (stream upTo: $<).
			stream peek = $<
				ifTrue: [writeStream nextPut:
					stream next]
				ifFalse:
					[stream skip: -1.
					done := true]].
	writeStream contents last = Character cr
		ifTrue: [writeStream skip: -1].
	^writeStream contents! !