'OBJECT-ORIENTED PROGRAMMING
Peter Coad / Jill Nicola
Source Code Diskette
(C) 1993  Object International, Inc.  All rights reserved.
'!

'LANVWCTR.ST'!

ModelValueViewContainer subclass: #LaneViewContainer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Traffic-Views'!


!LaneViewContainer methodsFor: 'build views'!

buildCautionLightFor: lane 
	^self
		makeLaneLightBox: lane
		color: self cautionColor
		rule: self cautionLightRule!

buildGoLightFor: lane 
	^self
		makeLaneLightBox: lane
		color: self goColor
		rule: self goLightRule!

buildStopLightFor: lane 
	^self
		makeLaneLightBox: lane
		color: self stopColor
		rule: self stopLightRule! !

!LaneViewContainer methodsFor: 'colors'!

cautionColor
	^ColorValue yellow!

goColor
	^ColorValue green!

stopColor
	^ColorValue red! !

!LaneViewContainer methodsFor: 'rules'!

cautionLightRule
	^[:lane | 0 < lane currentFlowRate 
			and: [lane currentFlowRate <
				lane maxFlowRate]]!

goLightRule
	^[:lane | lane currentFlowRate = lane maxFlowRate]!

stopLightRule
	^[:lane | lane currentFlowRate = 0]! !

!LaneViewContainer methodsFor: 'view layout'!

horizontalLightFor: lane 
	| redLight yellowLight greenLight |
	redLight := self buildStopLightFor: lane.
	yellowLight := self buildCautionLightFor: lane.
	greenLight := self buildGoLightFor: lane.
	self addView: redLight in: self leftArea.
	self addView: yellowLight in: self horizontalMiddleArea.
	self addView: greenLight in: self rightArea!

verticalLightFor: lane
	| redLight yellowLight greenLight |

	redLight := self buildStopLightFor: lane.
	yellowLight := self buildCautionLightFor: lane.
	greenLight := self buildGoLightFor: lane.

	self addView: redLight in: self topArea.
	self addView: yellowLight in: self verticalMiddleArea.
	self addView: greenLight in: self bottomArea! !

!LaneViewContainer methodsFor: 'view creation'!

makeLaneLightBox: lane color: colorValue rule: ruleBlock 
	| lightBox aShape |
	aShape := Circle center: 20@20 radius: 15.
	lightBox := super
			makeLightBoxFor: lane
			color: colorValue
			rule: ruleBlock
			shape: aShape.
	^lightBox! !

!LaneViewContainer methodsFor: 'window opening'!

openOn: lane
	self openVerticalLightOn: lane at: 50@ 50 label:
		'Traffic Lane'!

openVerticalLightOn: lane at: screenPt label: windowLabel
	| window |
	self verticalLightFor: lane.
	window := ScheduledWindow new.
	window component: self.
	window minimumSize: 50 @ 150.
	window label: windowLabel.
	window openDisplayAt: screenPt! !

!LaneViewContainer methodsFor: 'private-layout'!

bottomArea
	^self constraintFrame: 0 @ 2 / 3 corner: 1 @ 1!

horizontalMiddleArea
	^self constraintFrame: 1 / 3 @ 0 corner: 2 / 3 @ 1!

leftArea
	^self constraintFrame: 0 @ 0 corner: 1 / 3 @ 1!

rightArea
	^self constraintFrame: 2 / 3 @ 0 corner: 1 @ 1!

topArea
	^self constraintFrame: 0 @ 0 corner: 1 @ (1 / 3)!

verticalMiddleArea
	^self constraintFrame: 0 @ 1 / 3 corner: 1 @ (2 / 3)! !